from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.textlabels import PMVLabel
from reportlab.graphics.widgetbase import Widget
from reportlab.lib.attrmap import *
from reportlab.lib.validators import Validator

__version__: Final[str]

class AxisLabelAnnotation:
    def __init__(self, v, **kwds) -> None: ...
    def __call__(self, axis): ...

class AxisLineAnnotation:
    def __init__(self, v, **kwds) -> None: ...
    def __call__(self, axis): ...

class AxisBackgroundAnnotation:
    def __init__(self, colors, **kwds) -> None: ...
    def __call__(self, axis): ...

class TickLU:
    accuracy: Incomplete
    T: Incomplete
    def __init__(self, *T, **kwds) -> None: ...
    def __contains__(self, t) -> bool: ...
    def __getitem__(self, t): ...

class _AxisG(Widget):
    def makeGrid(self, g, dim=None, parent=None, exclude=[]) -> None: ...
    def getGridDims(self, start=None, end=None): ...
    @property
    def isYAxis(self): ...
    @property
    def isXAxis(self): ...
    def addAnnotations(self, g, A=None) -> None: ...
    def draw(self): ...

class CALabel(PMVLabel):
    def __init__(self, **kw) -> None: ...

class CategoryAxis(_AxisG):
    visible: int
    visibleAxis: int
    visibleTicks: int
    visibleLabels: int
    visibleGrid: int
    drawGridLast: bool
    strokeWidth: int
    strokeColor: Incomplete
    strokeDashArray: Incomplete
    gridStrokeLineJoin: Incomplete
    gridStrokeLineCap: Incomplete
    gridStrokeMiterLimit: Incomplete
    gridStrokeWidth: float
    gridStrokeColor: Incomplete
    gridStrokeDashArray: Incomplete
    gridStart: Incomplete
    strokeLineJoin: Incomplete
    strokeLineCap: Incomplete
    strokeMiterLimit: Incomplete
    labels: Incomplete
    categoryNames: Incomplete
    joinAxis: Incomplete
    joinAxisPos: Incomplete
    joinAxisMode: Incomplete
    labelAxisMode: str
    reverseDirection: int
    style: str
    tickShift: int
    loPad: int
    hiPad: int
    loLLen: int
    hiLLen: int
    def __init__(self) -> None: ...
    def setPosition(self, x, y, length) -> None: ...
    def configure(self, multiSeries, barWidth=None) -> None: ...
    def scale(self, idx): ...
    def midScale(self, idx): ...

class _XTicks:
    @property
    def actualTickStrokeWidth(self): ...
    @property
    def actualTickStrokeColor(self): ...
    def makeTicks(self): ...

class _YTicks(_XTicks):
    def makeTicks(self): ...

class XCategoryAxis(_XTicks, CategoryAxis):
    tickUp: int
    tickDown: int
    def __init__(self) -> None: ...
    categoryNames: Incomplete
    def demo(self): ...
    def joinToAxis(self, yAxis, mode: str = "bottom", pos=None) -> None: ...
    def loScale(self, idx): ...
    def makeAxis(self): ...
    def makeTickLabels(self): ...

class YCategoryAxis(_YTicks, CategoryAxis):
    tickLeft: int
    tickRight: int
    def __init__(self) -> None: ...
    categoryNames: Incomplete
    def demo(self): ...
    def joinToAxis(self, xAxis, mode: str = "left", pos=None) -> None: ...
    def loScale(self, idx): ...
    def makeAxis(self): ...
    def makeTickLabels(self): ...

class TickLabeller:
    def __call__(self, axis, value): ...

class ValueAxis(_AxisG):
    def __init__(self, **kw) -> None: ...
    def setPosition(self, x, y, length) -> None: ...
    def configure(self, dataSeries) -> None: ...
    def makeTickLabels(self): ...
    def scale(self, value): ...

class XValueAxis(_XTicks, ValueAxis):
    tickUp: int
    tickDown: int
    joinAxis: Incomplete
    joinAxisMode: Incomplete
    joinAxisPos: Incomplete
    def __init__(self, **kw) -> None: ...
    def demo(self): ...
    def joinToAxis(self, yAxis, mode: str = "bottom", pos=None) -> None: ...
    def makeAxis(self): ...

def parseDayAndMonth(dmstr): ...

class _isListOfDaysAndMonths(Validator):
    def test(self, x): ...
    def normalize(self, x): ...

isListOfDaysAndMonths: Incomplete

class NormalDateXValueAxis(XValueAxis):
    bottomAxisLabelSlack: float
    niceMonth: int
    forceEndDate: int
    forceFirstDate: int
    forceDatesEachYear: Incomplete
    dailyFreq: int
    xLabelFormat: str
    dayOfWeekName: Incomplete
    monthName: Incomplete
    specialTickClear: int
    valueSteps: Incomplete
    def __init__(self, **kw) -> None: ...
    def configure(self, data) -> None: ...

class YValueAxis(_YTicks, ValueAxis):
    tickRight: int
    tickLeft: int
    joinAxis: Incomplete
    joinAxisMode: Incomplete
    joinAxisPos: Incomplete
    def __init__(self) -> None: ...
    def demo(self): ...
    def joinToAxis(self, xAxis, mode: str = "left", pos=None) -> None: ...
    def makeAxis(self): ...

class TimeValueAxis:
    labelTextFormat: Incomplete
    def __init__(self, *args, **kwds) -> None: ...
    def timeLabelTextFormatter(self, val): ...

class XTimeValueAxis(TimeValueAxis, XValueAxis):
    def __init__(self, *args, **kwds) -> None: ...

class AdjYValueAxis(YValueAxis):
    requiredRange: int
    leftAxisPercent: int
    leftAxisOrigShiftIPC: float
    leftAxisOrigShiftMin: int
    leftAxisSkipLL0: int
    valueSteps: Incomplete
    def __init__(self, **kw) -> None: ...

class LogValueAxis(ValueAxis): ...

class LogAxisTickLabeller(TickLabeller):
    def __call__(self, axis, value): ...

class LogAxisTickLabellerS(TickLabeller):
    def __call__(self, axis, value): ...

class LogAxisLabellingSetup:
    labels: Incomplete
    labelTextFormat: Incomplete
    def __init__(self) -> None: ...

class LogXValueAxis(LogValueAxis, LogAxisLabellingSetup, XValueAxis):
    def __init__(self) -> None: ...
    def scale(self, value): ...

class LogYValueAxis(LogValueAxis, LogAxisLabellingSetup, YValueAxis):
    def __init__(self) -> None: ...
    def scale(self, value): ...



from collections.abc import Container, Iterable
from typing_extensions import TypeAlias

from .callbacks import _Callback
from .css_sanitizer import CSSSanitizer
from .linkifier import DEFAULT_CALLBACKS as DEFAULT_CALLBACKS, Linker as Linker
from .sanitizer import (
    ALLOWED_ATTRIBUTES as ALLOWED_ATTRIBUTES,
    ALLOWED_PROTOCOLS as ALLOWED_PROTOCOLS,
    ALLOWED_TAGS as ALLOWED_TAGS,
    Cleaner as Cleaner,
    _Attributes,
)

__all__ = ["clean", "linkify"]

__releasedate__: str
__version__: str

_HTMLAttrKey: TypeAlias = tuple[str | None, str]  # noqa: Y047

def clean(
    text: str,
    tags: Iterable[str] = ...,
    attributes: _Attributes = ...,
    protocols: Iterable[str] = ...,
    strip: bool = False,
    strip_comments: bool = True,
    css_sanitizer: CSSSanitizer | None = None,
) -> str: ...
def linkify(
    text: str, callbacks: Iterable[_Callback] = ..., skip_tags: Container[str] | None = None, parse_email: bool = False
) -> str: ...



from collections.abc import Iterable, Mapping
from typing import Literal
from xml.etree.ElementTree import Element, TreeBuilder

from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension
from markdown.htmlparser import HTMLExtractor
from markdown.postprocessors import RawHtmlPostprocessor
from markdown.preprocessors import Preprocessor

class HTMLExtractorExtra(HTMLExtractor):
    block_level_tags: set[str]
    span_tags: set[str]
    raw_tags: set[str]
    block_tags: set[str]
    span_and_blocks_tags: set[str]
    mdstack: list[str]
    treebuilder: TreeBuilder
    mdstate: list[Literal["block", "span", "off"] | None]
    mdstarted: list[bool]
    def get_element(self) -> Element: ...
    def get_state(self, tag: str, attrs: Mapping[str, str]) -> Literal["block", "span", "off"] | None: ...
    def handle_starttag(self, tag: str, attrs: Iterable[tuple[str, str | None]]) -> None: ...
    def handle_endtag(self, tag: str) -> None: ...
    def handle_startendtag(self, tag: str, attrs: Iterable[tuple[str, str | None]]) -> None: ...
    def handle_data(self, data: str) -> None: ...
    def handle_empty_tag(self, data: str, is_block: bool) -> None: ...
    def parse_pi(self, i: int) -> int: ...
    def parse_html_declaration(self, i: int) -> int: ...

class HtmlBlockPreprocessor(Preprocessor): ...

class MarkdownInHtmlProcessor(BlockProcessor):
    def parse_element_content(self, element: Element) -> None: ...

class MarkdownInHTMLPostprocessor(RawHtmlPostprocessor):
    def stash_to_string(self, text: str | Element) -> str: ...

class MarkdownInHtmlExtension(Extension): ...

def makeExtension(**kwargs) -> MarkdownInHtmlExtension: ...



import re
from typing import Final

DIGIT: Final[str]
ALPHA: Final[str]
HEXDIG: Final[str]
pct_encoded: Final[str]
unreserved: Final[str]
gen_delims: Final[str]
sub_delims: Final[str]
pchar: Final[str]
reserved: Final[str]
scheme: Final[str]
dec_octet: Final[str]
IPv4address: Final[str]
IPv6address: Final[str]
IPvFuture: Final[str]
IP_literal: Final[str]
reg_name: Final[str]
userinfo: Final[str]
host: Final[str]
port: Final[str]
authority: Final[str]
segment: Final[str]
segment_nz: Final[str]
segment_nz_nc: Final[str]
path_abempty: Final[str]
path_absolute: Final[str]
path_noscheme: Final[str]
path_rootless: Final[str]
path_empty: Final[str]
path: Final[str]
query: Final[str]
fragment: Final[str]
hier_part: Final[str]
relative_part: Final[str]
relative_ref: Final[str]
URI: Final[str]
URI_reference: Final[str]
absolute_URI: Final[str]

def is_uri(uri: str) -> re.Match[str] | None: ...
def is_uri_reference(uri: str) -> re.Match[str] | None: ...
def is_absolute_uri(uri: str) -> re.Match[str] | None: ...



from .patcher import patch as patch, patch_all as patch_all
from .recorder import AWSXRayRecorder as AWSXRayRecorder

xray_recorder: AWSXRayRecorder

__all__ = ["patch", "patch_all", "xray_recorder", "AWSXRayRecorder"]



from braintree.search import Search

class CreditCardVerificationSearch:
    credit_card_cardholder_name: Search.TextNodeBuilder
    id: Search.TextNodeBuilder
    credit_card_expiration_date: Search.EqualityNodeBuilder
    credit_card_number: Search.PartialMatchNodeBuilder
    credit_card_card_type: Search.MultipleValueNodeBuilder
    ids: Search.MultipleValueNodeBuilder
    created_at: Search.RangeNodeBuilder
    status: Search.MultipleValueNodeBuilder
    billing_postal_code: Search.TextNodeBuilder
    customer_email: Search.TextNodeBuilder
    customer_id: Search.TextNodeBuilder
    payment_method_token: Search.TextNodeBuilder



from _typeshed import Incomplete
from collections.abc import Callable
from typing import Any, Protocol, TypeVar, type_check_only

_T = TypeVar("_T")

# at runtime, socketio.namespace.BaseNamespace, but socketio isn't py.typed
@type_check_only
class _BaseNamespace(Protocol):
    def is_asyncio_based(self) -> bool: ...
    def trigger_event(self, event: str, *args): ...

# at runtime, socketio.namespace.BaseNamespace, but socketio isn't py.typed
class _Namespace(_BaseNamespace, Protocol):
    def emit(
        self,
        event: str,
        data=None,
        to=None,
        room: str | None = None,
        skip_sid=None,
        namespace: str | None = None,
        callback: Callable[..., Incomplete] | None = None,
        ignore_queue: bool = False,
    ): ...
    def send(
        self,
        data,
        to=None,
        room: str | None = None,
        skip_sid=None,
        namespace: str | None = None,
        callback: Callable[..., Incomplete] | None = None,
        ignore_queue: bool = False,
    ) -> None: ...
    def call(
        self, event: str, data=None, to=None, sid=None, namespace: str | None = None, timeout=None, ignore_queue: bool = False
    ): ...
    def enter_room(self, sid, room: str, namespace: str | None = None): ...
    def leave_room(self, sid, room: str, namespace: str | None = None): ...
    def close_room(self, room: str, namespace: str | None = None): ...
    def rooms(self, sid, namespace: str | None = None): ...
    def get_session(self, sid, namespace: str | None = None): ...
    def save_session(self, sid, session, namespace: str | None = None): ...
    def session(self, sid, namespace: str | None = None): ...
    def disconnect(self, sid, namespace: str | None = None): ...

class Namespace(_Namespace):
    def __init__(self, namespace: str | None = None) -> None: ...
    def trigger_event(self, event: str, *args): ...
    def emit(  # type: ignore[override]
        self,
        event: str,
        data=None,
        room: str | None = None,
        include_self: bool = True,
        namespace: str | None = None,
        callback: Callable[..., _T] | None = None,
    ) -> _T | tuple[str, int]: ...
    def send(  # type: ignore[override]
        self,
        data,
        room: str | None = None,
        include_self: bool = True,
        namespace: str | None = None,
        callback: Callable[..., Any] | None = None,
    ) -> None: ...
    def close_room(self, room: str, namespace: str | None = None) -> None: ...



from pika.adapters.base_connection import BaseConnection as BaseConnection
from pika.adapters.blocking_connection import BlockingConnection as BlockingConnection
from pika.adapters.select_connection import IOLoop as IOLoop, SelectConnection as SelectConnection



from collections.abc import MutableMapping
from typing import TypeVar

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

class CaseInsensitiveDict(MutableMapping[_KT, _VT]):
    def __init__(self, other=None, **kwargs) -> None: ...
    def __contains__(self, item): ...
    def __delitem__(self, key) -> None: ...
    def __setitem__(self, key, item) -> None: ...
    def __getitem__(self, key): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def keys(self): ...
    def values(self): ...
    def items(self): ...
    def __eq__(self, other): ...
    def copy(self): ...

class CaseInsensitiveWithAliasDict(CaseInsensitiveDict[_KT, _VT]):
    def __init__(self, other=None, **kwargs) -> None: ...
    def aliases(self): ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...
    def set_alias(self, key, alias, ignore_duplicates: bool = False) -> None: ...
    def remove_alias(self, alias) -> None: ...
    def __getitem__(self, key): ...
    def copy(self): ...



import logging

LOGGER_KEY: str
DEFAULT_LEVEL: int
PY2: bool
text_type: type

class ColorFormatter(logging.Formatter):
    colors: dict[str, dict[str, str]]
    def format(self, record: logging.LogRecord) -> str: ...

class ClickHandler(logging.Handler):
    def emit(self, record: logging.LogRecord) -> None: ...

def basic_config(logger: logging.Logger | str | None = None) -> None: ...



from _typeshed import Incomplete

class TestingGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def make_past_due(self, subscription_id, number_of_days_past_due: int = 1) -> None: ...
    def settle_transaction(self, transaction_id): ...
    def settlement_confirm_transaction(self, transaction_id): ...
    def settlement_decline_transaction(self, transaction_id): ...
    def settlement_pending_transaction(self, transaction_id): ...
    def create_3ds_verification(self, merchant_account_id, params): ...



from typing import Any
from typing_extensions import Self

__tracebackhide__: bool

class DictMixin:
    def contains_key(self, *keys: Any) -> Self: ...
    def does_not_contain_key(self, *keys: Any) -> Self: ...
    def contains_value(self, *values: Any) -> Self: ...
    def does_not_contain_value(self, *values: Any) -> Self: ...
    def contains_entry(self, *args: Any, **kwargs: dict[str, Any]) -> Self: ...
    def does_not_contain_entry(self, *args: Any, **kwargs: dict[str, Any]) -> Self: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete, Unused
from collections.abc import Iterator
from re import Pattern
from typing import ClassVar, Final, Literal, TypeVar, overload
from typing_extensions import Self

from openpyxl.descriptors import Strict, Typed
from openpyxl.descriptors.base import _N, Bool, Integer, MinMax, String, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable

_S = TypeVar("_S", bound=Serialisable)

COLOR_INDEX: Final[tuple[str, ...]]
BLACK: Final = "00000000"
WHITE: Final = "00FFFFFF"
BLUE: Final = "00FFFFFF"
aRGB_REGEX: Final[Pattern[str]]

class RGB(Typed[str, _N]):
    expected_type: type[str]
    @overload
    def __init__(self: RGB[Literal[True]], name: str | None = None, *, allow_none: Literal[True]) -> None: ...
    @overload
    def __init__(self: RGB[Literal[False]], name: str | None = None, *, allow_none: Literal[False] = False) -> None: ...
    @overload
    def __set__(self: RGB[Literal[True]], instance: Serialisable | Strict, value: str | None) -> None: ...
    @overload
    def __set__(self: RGB[Literal[False]], instance: Serialisable | Strict, value: str) -> None: ...

class Color(Serialisable):
    tagname: ClassVar[str]
    rgb: RGB[Literal[False]]
    indexed: Integer[Literal[False]]
    auto: Bool[Literal[False]]
    theme: Integer[Literal[False]]
    tint: MinMax[float, Literal[False]]
    type: String[Literal[False]]
    def __init__(
        self,
        rgb="00000000",
        indexed: ConvertibleToInt | None = None,
        auto: _ConvertibleToBool | None = None,
        theme: ConvertibleToInt | None = None,
        tint: ConvertibleToFloat = 0.0,
        index: ConvertibleToInt | None = None,
        type: Unused = "rgb",
    ) -> None: ...
    @property
    def value(self) -> str | int | bool: ...
    @value.setter
    def value(self, value: str | ConvertibleToInt | _ConvertibleToBool) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...
    @property
    def index(self) -> str | int | bool: ...
    @overload
    def __add__(self, other: Color) -> Self: ...
    @overload
    def __add__(self, other: _S) -> _S: ...

class ColorDescriptor(Typed[Color, _N]):
    expected_type: type[Color]
    @overload
    def __init__(self: ColorDescriptor[Literal[True]], name: str | None = None, *, allow_none: Literal[True]) -> None: ...
    @overload
    def __init__(
        self: ColorDescriptor[Literal[False]], name: str | None = None, *, allow_none: Literal[False] = False
    ) -> None: ...
    @overload
    def __set__(self: ColorDescriptor[Literal[True]], instance: Serialisable | Strict, value: str | Color | None) -> None: ...
    @overload
    def __set__(self: ColorDescriptor[Literal[False]], instance: Serialisable | Strict, value: str | Color) -> None: ...

class RgbColor(Serialisable):
    tagname: ClassVar[str]
    rgb: RGB[Literal[False]]
    def __init__(self, rgb: str) -> None: ...

class ColorList(Serialisable):
    tagname: ClassVar[str]
    indexedColors: Incomplete
    mruColors: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self, indexedColors: list[RgbColor] | tuple[RgbColor, ...] = (), mruColors: list[Color] | tuple[Color, ...] = ()
    ) -> None: ...
    def __bool__(self) -> bool: ...
    @property
    def index(self) -> list[str]: ...



from _typeshed import Incomplete, SupportsGetItem

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["percolation_centrality"]

@_dispatchable
def percolation_centrality(
    G: Graph[_Node],
    attribute: str | None = "percolation",
    states: SupportsGetItem[Incomplete, Incomplete] | None = None,
    weight: str | None = None,
) -> dict[Incomplete, float]: ...



from collections.abc import Iterable
from typing import TypeVar, overload

from networkx import MultiGraph
from networkx.classes.graph import Graph
from networkx.utils.misc import _RandomState

_G = TypeVar("_G", bound=Graph[int])
__all__ = ["random_clustered_graph"]

@overload
def random_clustered_graph(
    joint_degree_sequence: Iterable[tuple[int, int]], create_using: None = None, seed: _RandomState = None
) -> MultiGraph[int]: ...
@overload
def random_clustered_graph(
    joint_degree_sequence: Iterable[tuple[int, int]], create_using: type[_G], seed: _RandomState = None
) -> _G: ...



from pyasn1.type.namedtype import NamedTypes
from pyasn1.type.tag import TagSet
from pyasn1.type.univ import OctetString, Sequence

class UserIdentity(OctetString):
    tagSet: TagSet
    encoding: str

class OldPasswd(OctetString):
    tagSet: TagSet
    encoding: str

class NewPasswd(OctetString):
    tagSet: TagSet
    encoding: str

class GenPasswd(OctetString):
    tagSet: TagSet
    encoding: str

class PasswdModifyRequestValue(Sequence):
    componentType: NamedTypes

class PasswdModifyResponseValue(Sequence):
    componentType: NamedTypes



from _typeshed import Incomplete
from html.parser import HTMLParser

from reportlab.lib.abag import ABag

__all__ = ("ParaFrag", "ParaParser")

class _PCT(float):
    def __new__(cls, v): ...
    def normalizedValue(self, normalizer): ...
    def __copy__(self): ...
    def __deepcopy__(self, mem): ...

class _ExValidate:
    tag: Incomplete
    attr: Incomplete
    def __init__(self, tag, attr) -> None: ...
    def invalid(self, s) -> None: ...
    def validate(self, parser, s): ...
    def __call__(self, parser, s): ...

class _CheckSup(_ExValidate):
    fontSize: Incomplete
    def validate(self, parser, s): ...
    def __call__(self, parser, s): ...

class _CheckUS(_ExValidate):
    def validate(self, parser, s): ...

class ParaFrag(ABag): ...

class ParaParser(HTMLParser):
    def __getattr__(self, attrName): ...
    def start_b(self, attributes) -> None: ...
    def end_b(self) -> None: ...
    def start_strong(self, attributes) -> None: ...
    def end_strong(self) -> None: ...
    def start_i(self, attributes) -> None: ...
    def end_i(self) -> None: ...
    def start_em(self, attributes) -> None: ...
    def end_em(self) -> None: ...
    def start_u(self, attributes) -> None: ...
    def end_u(self) -> None: ...
    def start_strike(self, attributes) -> None: ...
    def end_strike(self) -> None: ...
    def start_link(self, attributes) -> None: ...
    def end_link(self) -> None: ...
    def start_a(self, attributes) -> None: ...
    def end_a(self) -> None: ...
    def start_img(self, attributes) -> None: ...
    def end_img(self) -> None: ...
    def start_super(self, attributes) -> None: ...
    def end_super(self) -> None: ...
    start_sup = start_super
    end_sup = end_super
    def start_sub(self, attributes) -> None: ...
    def end_sub(self) -> None: ...
    def start_nobr(self, attrs) -> None: ...
    def end_nobr(self) -> None: ...
    def handle_charref(self, name) -> None: ...
    def syntax_error(self, lineno, message) -> None: ...
    def start_greek(self, attr) -> None: ...
    def end_greek(self) -> None: ...
    def start_unichar(self, attr) -> None: ...
    def end_unichar(self) -> None: ...
    def start_font(self, attr) -> None: ...
    def end_font(self) -> None: ...
    def start_span(self, attr) -> None: ...
    def end_span(self) -> None: ...
    def start_br(self, attr) -> None: ...
    def end_br(self) -> None: ...
    def start_para(self, attr) -> None: ...
    def end_para(self) -> None: ...
    bFragList: Incomplete
    def start_bullet(self, attr) -> None: ...
    def end_bullet(self) -> None: ...
    def start_seqdefault(self, attr) -> None: ...
    def end_seqdefault(self) -> None: ...
    def start_seqreset(self, attr) -> None: ...
    def end_seqreset(self) -> None: ...
    def start_seqchain(self, attr) -> None: ...
    end_seqchain = end_seqreset
    def start_seqformat(self, attr) -> None: ...
    end_seqformat = end_seqreset
    start_seqDefault = start_seqdefault
    end_seqDefault = end_seqdefault
    start_seqReset = start_seqreset
    end_seqReset = end_seqreset
    start_seqChain = start_seqchain
    end_seqChain = end_seqchain
    start_seqFormat = start_seqformat
    end_seqFormat = end_seqformat
    def start_seq(self, attr) -> None: ...
    def end_seq(self) -> None: ...
    def start_ondraw(self, attr) -> None: ...
    start_onDraw = start_ondraw
    end_onDraw = end_seq
    end_ondraw = end_seq
    def start_index(self, attr) -> None: ...
    end_index = end_seq
    def start_unknown(self, attr) -> None: ...
    end_unknown = end_seq
    def getAttributes(self, attr, attrMap): ...
    verbose: Incomplete
    caseSensitive: Incomplete
    ignoreUnknownTags: Incomplete
    def __init__(
        self, verbose: int = 0, caseSensitive: int = 0, ignoreUnknownTags: int = 1, crashOnError: bool = True
    ) -> None: ...
    def handle_data(self, data) -> None: ...
    def handle_cdata(self, data) -> None: ...
    def tt_parse(self, tt, style): ...
    def findSpanStyle(self, style) -> None: ...
    def parse(self, text, style): ...
    def handle_starttag(self, tag, attrs) -> None: ...
    def handle_endtag(self, tag) -> None: ...
    def handle_entityref(self, name) -> None: ...



import _typeshed
import sys
from _typeshed import ReadableBuffer, StrOrBytesPath, WriteableBuffer
from abc import abstractmethod
from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
from ctypes import CDLL, ArgumentError as ArgumentError, c_void_p
from types import GenericAlias
from typing import Any, ClassVar, Final, Generic, TypeVar, final, overload, type_check_only
from typing_extensions import Self, TypeAlias

_T = TypeVar("_T")
_CT = TypeVar("_CT", bound=_CData)

FUNCFLAG_CDECL: Final = 0x1
FUNCFLAG_PYTHONAPI: Final = 0x4
FUNCFLAG_USE_ERRNO: Final = 0x8
FUNCFLAG_USE_LASTERROR: Final = 0x10
RTLD_GLOBAL: Final[int]
RTLD_LOCAL: Final[int]

if sys.version_info >= (3, 11):
    CTYPES_MAX_ARGCOUNT: Final[int]

if sys.version_info >= (3, 12):
    SIZEOF_TIME_T: Final[int]

if sys.platform == "win32":
    # Description, Source, HelpFile, HelpContext, scode
    _COMError_Details: TypeAlias = tuple[str | None, str | None, str | None, int | None, int | None]

    class COMError(Exception):
        hresult: int
        text: str | None
        details: _COMError_Details

        def __init__(self, hresult: int, text: str | None, details: _COMError_Details) -> None: ...

    def CopyComPointer(src: _PointerLike, dst: _PointerLike | _CArgObject) -> int: ...

    FUNCFLAG_HRESULT: Final = 0x2
    FUNCFLAG_STDCALL: Final = 0x0

    def FormatError(code: int = ...) -> str: ...
    def get_last_error() -> int: ...
    def set_last_error(value: int) -> int: ...
    def LoadLibrary(name: str, load_flags: int = 0, /) -> int: ...
    def FreeLibrary(handle: int, /) -> None: ...

else:
    def dlclose(handle: int, /) -> None: ...
    # The default for flag is RTLD_GLOBAL|RTLD_LOCAL, which is platform dependent.
    def dlopen(name: StrOrBytesPath, flag: int = ..., /) -> int: ...
    def dlsym(handle: int, name: str, /) -> int: ...

if sys.version_info >= (3, 13):
    # This class is not exposed. It calls itself _ctypes.CType_Type.
    @type_check_only
    class _CType_Type(type):
        # By default mypy complains about the following two methods, because strictly speaking cls
        # might not be a Type[_CT]. However this doesn't happen because this is only a
        # metaclass for subclasses of _CData.
        def __mul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

    _CTypeBaseType = _CType_Type

else:
    _CTypeBaseType = type

# This class is not exposed.
@type_check_only
class _CData:
    _b_base_: int
    _b_needsfree_: bool
    _objects: Mapping[Any, int] | None
    def __buffer__(self, flags: int, /) -> memoryview: ...
    def __ctypes_from_outparam__(self, /) -> Self: ...
    if sys.version_info >= (3, 14):
        __pointer_type__: type

# this is a union of all the subclasses of _CData, which is useful because of
# the methods that are present on each of those subclasses which are not present
# on _CData itself.
_CDataType: TypeAlias = _SimpleCData[Any] | _Pointer[Any] | CFuncPtr | Union | Structure | Array[Any]

# This class is not exposed. It calls itself _ctypes.PyCSimpleType.
@type_check_only
class _PyCSimpleType(_CTypeBaseType):
    def from_address(self: type[_typeshed.Self], value: int, /) -> _typeshed.Self: ...
    def from_buffer(self: type[_typeshed.Self], obj: WriteableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_buffer_copy(self: type[_typeshed.Self], buffer: ReadableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_param(self: type[_typeshed.Self], value: Any, /) -> _typeshed.Self | _CArgObject: ...
    def in_dll(self: type[_typeshed.Self], dll: CDLL, name: str, /) -> _typeshed.Self: ...
    if sys.version_info < (3, 13):
        # Inherited from CType_Type starting on 3.13
        def __mul__(self: type[_CT], value: int, /) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(self: type[_CT], value: int, /) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class _SimpleCData(_CData, Generic[_T], metaclass=_PyCSimpleType):
    value: _T
    # The TypeVar can be unsolved here,
    # but we can't use overloads without creating many, many mypy false-positive errors
    def __init__(self, value: _T = ...) -> None: ...  # pyright: ignore[reportInvalidTypeVarUse]
    def __ctypes_from_outparam__(self, /) -> _T: ...  # type: ignore[override]

@type_check_only
class _CanCastTo(_CData): ...

@type_check_only
class _PointerLike(_CanCastTo): ...

# This type is not exposed. It calls itself _ctypes.PyCPointerType.
@type_check_only
class _PyCPointerType(_CTypeBaseType):
    def from_address(self: type[_typeshed.Self], value: int, /) -> _typeshed.Self: ...
    def from_buffer(self: type[_typeshed.Self], obj: WriteableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_buffer_copy(self: type[_typeshed.Self], buffer: ReadableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_param(self: type[_typeshed.Self], value: Any, /) -> _typeshed.Self | _CArgObject: ...
    def in_dll(self: type[_typeshed.Self], dll: CDLL, name: str, /) -> _typeshed.Self: ...
    def set_type(self, type: _CTypeBaseType, /) -> None: ...
    if sys.version_info < (3, 13):
        # Inherited from CType_Type starting on 3.13
        def __mul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class _Pointer(_PointerLike, _CData, Generic[_CT], metaclass=_PyCPointerType):
    _type_: type[_CT]
    contents: _CT
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg: _CT) -> None: ...
    @overload
    def __getitem__(self, key: int, /) -> Any: ...
    @overload
    def __getitem__(self, key: slice, /) -> list[Any]: ...
    def __setitem__(self, key: int, value: Any, /) -> None: ...

if sys.version_info < (3, 14):
    @overload
    def POINTER(type: None, /) -> type[c_void_p]: ...
    @overload
    def POINTER(type: type[_CT], /) -> type[_Pointer[_CT]]: ...
    def pointer(obj: _CT, /) -> _Pointer[_CT]: ...

# This class is not exposed. It calls itself _ctypes.CArgObject.
@final
@type_check_only
class _CArgObject: ...

if sys.version_info >= (3, 14):
    def byref(obj: _CData | _CDataType, offset: int = 0, /) -> _CArgObject: ...

else:
    def byref(obj: _CData | _CDataType, offset: int = 0) -> _CArgObject: ...

_ECT: TypeAlias = Callable[[_CData | _CDataType | None, CFuncPtr, tuple[_CData | _CDataType, ...]], _CDataType]
_PF: TypeAlias = tuple[int] | tuple[int, str | None] | tuple[int, str | None, Any]

# This class is not exposed. It calls itself _ctypes.PyCFuncPtrType.
@type_check_only
class _PyCFuncPtrType(_CTypeBaseType):
    def from_address(self: type[_typeshed.Self], value: int, /) -> _typeshed.Self: ...
    def from_buffer(self: type[_typeshed.Self], obj: WriteableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_buffer_copy(self: type[_typeshed.Self], buffer: ReadableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_param(self: type[_typeshed.Self], value: Any, /) -> _typeshed.Self | _CArgObject: ...
    def in_dll(self: type[_typeshed.Self], dll: CDLL, name: str, /) -> _typeshed.Self: ...
    if sys.version_info < (3, 13):
        # Inherited from CType_Type starting on 3.13
        def __mul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class CFuncPtr(_PointerLike, _CData, metaclass=_PyCFuncPtrType):
    restype: type[_CDataType] | Callable[[int], Any] | None
    argtypes: Sequence[type[_CDataType]]
    errcheck: _ECT
    # Abstract attribute that must be defined on subclasses
    _flags_: ClassVar[int]
    @overload
    def __new__(cls) -> Self: ...
    @overload
    def __new__(cls, address: int, /) -> Self: ...
    @overload
    def __new__(cls, callable: Callable[..., Any], /) -> Self: ...
    @overload
    def __new__(cls, func_spec: tuple[str | int, CDLL], paramflags: tuple[_PF, ...] | None = ..., /) -> Self: ...
    if sys.platform == "win32":
        @overload
        def __new__(
            cls, vtbl_index: int, name: str, paramflags: tuple[_PF, ...] | None = ..., iid: _CData | _CDataType | None = ..., /
        ) -> Self: ...

    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...

_GetT = TypeVar("_GetT")
_SetT = TypeVar("_SetT")

# This class is not exposed. It calls itself _ctypes.CField.
@final
@type_check_only
class _CField(Generic[_CT, _GetT, _SetT]):
    offset: int
    size: int
    if sys.version_info >= (3, 10):
        @overload
        def __get__(self, instance: None, owner: type[Any] | None = None, /) -> Self: ...
        @overload
        def __get__(self, instance: Any, owner: type[Any] | None = None, /) -> _GetT: ...
    else:
        @overload
        def __get__(self, instance: None, owner: type[Any] | None, /) -> Self: ...
        @overload
        def __get__(self, instance: Any, owner: type[Any] | None, /) -> _GetT: ...

    def __set__(self, instance: Any, value: _SetT, /) -> None: ...

# This class is not exposed. It calls itself _ctypes.UnionType.
@type_check_only
class _UnionType(_CTypeBaseType):
    def from_address(self: type[_typeshed.Self], value: int, /) -> _typeshed.Self: ...
    def from_buffer(self: type[_typeshed.Self], obj: WriteableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_buffer_copy(self: type[_typeshed.Self], buffer: ReadableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_param(self: type[_typeshed.Self], value: Any, /) -> _typeshed.Self | _CArgObject: ...
    def in_dll(self: type[_typeshed.Self], dll: CDLL, name: str, /) -> _typeshed.Self: ...
    # At runtime, various attributes are created on a Union subclass based
    # on its _fields_. This method doesn't exist, but represents those
    # dynamically created attributes.
    def __getattr__(self, name: str) -> _CField[Any, Any, Any]: ...
    if sys.version_info < (3, 13):
        # Inherited from CType_Type starting on 3.13
        def __mul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Union(_CData, metaclass=_UnionType):
    _fields_: ClassVar[Sequence[tuple[str, type[_CDataType]] | tuple[str, type[_CDataType], int]]]
    _pack_: ClassVar[int]
    _anonymous_: ClassVar[Sequence[str]]
    if sys.version_info >= (3, 13):
        _align_: ClassVar[int]

    def __init__(self, *args: Any, **kw: Any) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...

# This class is not exposed. It calls itself _ctypes.PyCStructType.
@type_check_only
class _PyCStructType(_CTypeBaseType):
    def from_address(self: type[_typeshed.Self], value: int, /) -> _typeshed.Self: ...
    def from_buffer(self: type[_typeshed.Self], obj: WriteableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_buffer_copy(self: type[_typeshed.Self], buffer: ReadableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_param(self: type[_typeshed.Self], value: Any, /) -> _typeshed.Self | _CArgObject: ...
    def in_dll(self: type[_typeshed.Self], dll: CDLL, name: str, /) -> _typeshed.Self: ...
    # At runtime, various attributes are created on a Structure subclass based
    # on its _fields_. This method doesn't exist, but represents those
    # dynamically created attributes.
    def __getattr__(self, name: str) -> _CField[Any, Any, Any]: ...
    if sys.version_info < (3, 13):
        # Inherited from CType_Type starting on 3.13
        def __mul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Structure(_CData, metaclass=_PyCStructType):
    _fields_: ClassVar[Sequence[tuple[str, type[_CDataType]] | tuple[str, type[_CDataType], int]]]
    _pack_: ClassVar[int]
    _anonymous_: ClassVar[Sequence[str]]
    if sys.version_info >= (3, 13):
        _align_: ClassVar[int]

    def __init__(self, *args: Any, **kw: Any) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...

# This class is not exposed. It calls itself _ctypes.PyCArrayType.
@type_check_only
class _PyCArrayType(_CTypeBaseType):
    def from_address(self: type[_typeshed.Self], value: int, /) -> _typeshed.Self: ...
    def from_buffer(self: type[_typeshed.Self], obj: WriteableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_buffer_copy(self: type[_typeshed.Self], buffer: ReadableBuffer, offset: int = 0, /) -> _typeshed.Self: ...
    def from_param(self: type[_typeshed.Self], value: Any, /) -> _typeshed.Self | _CArgObject: ...
    def in_dll(self: type[_typeshed.Self], dll: CDLL, name: str, /) -> _typeshed.Self: ...
    if sys.version_info < (3, 13):
        # Inherited from CType_Type starting on 3.13
        def __mul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]
        def __rmul__(cls: type[_CT], other: int) -> type[Array[_CT]]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Array(_CData, Generic[_CT], metaclass=_PyCArrayType):
    @property
    @abstractmethod
    def _length_(self) -> int: ...
    @_length_.setter
    def _length_(self, value: int) -> None: ...
    @property
    @abstractmethod
    def _type_(self) -> type[_CT]: ...
    @_type_.setter
    def _type_(self, value: type[_CT]) -> None: ...
    # Note: only available if _CT == c_char
    @property
    def raw(self) -> bytes: ...
    @raw.setter
    def raw(self, value: ReadableBuffer) -> None: ...
    value: Any  # Note: bytes if _CT == c_char, str if _CT == c_wchar, unavailable otherwise
    # TODO: These methods cannot be annotated correctly at the moment.
    # All of these "Any"s stand for the array's element type, but it's not possible to use _CT
    # here, because of a special feature of ctypes.
    # By default, when accessing an element of an Array[_CT], the returned object has type _CT.
    # However, when _CT is a "simple type" like c_int, ctypes automatically "unboxes" the object
    # and converts it to the corresponding Python primitive. For example, when accessing an element
    # of an Array[c_int], a Python int object is returned, not a c_int.
    # This behavior does *not* apply to subclasses of "simple types".
    # If MyInt is a subclass of c_int, then accessing an element of an Array[MyInt] returns
    # a MyInt, not an int.
    # This special behavior is not easy to model in a stub, so for now all places where
    # the array element type would belong are annotated with Any instead.
    def __init__(self, *args: Any) -> None: ...
    @overload
    def __getitem__(self, key: int, /) -> Any: ...
    @overload
    def __getitem__(self, key: slice, /) -> list[Any]: ...
    @overload
    def __setitem__(self, key: int, value: Any, /) -> None: ...
    @overload
    def __setitem__(self, key: slice, value: Iterable[Any], /) -> None: ...
    def __iter__(self) -> Iterator[Any]: ...
    # Can't inherit from Sized because the metaclass conflict between
    # Sized and _CData prevents using _CDataMeta.
    def __len__(self) -> int: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

def addressof(obj: _CData | _CDataType, /) -> int: ...
def alignment(obj_or_type: _CData | _CDataType | type[_CData | _CDataType], /) -> int: ...
def get_errno() -> int: ...
def resize(obj: _CData | _CDataType, size: int, /) -> None: ...
def set_errno(value: int, /) -> int: ...
def sizeof(obj_or_type: _CData | _CDataType | type[_CData | _CDataType], /) -> int: ...
def PyObj_FromPtr(address: int, /) -> Any: ...
def Py_DECREF(o: _T, /) -> _T: ...
def Py_INCREF(o: _T, /) -> _T: ...
def buffer_info(o: _CData | _CDataType | type[_CData | _CDataType], /) -> tuple[str, int, tuple[int, ...]]: ...
def call_cdeclfunction(address: int, arguments: tuple[Any, ...], /) -> Any: ...
def call_function(address: int, arguments: tuple[Any, ...], /) -> Any: ...



from typing import Any
from typing_extensions import Self

class Settings:
    def __new__(cls, *args, **kw) -> Self: ...
    def __init__(self, settings: dict[str, Any] | None = None) -> None: ...
    @classmethod
    def get_key(cls, settings: dict[str, Any] | None = None) -> str: ...
    def replace(self, mod_settings: dict[str, Any] | None = None, **kwds) -> Self: ...

settings: Settings

def apply_settings(f): ...

class SettingValidationError(ValueError): ...

def check_settings(settings: Settings) -> None: ...



from _typeshed import Incomplete
from typing import IO, Final

from reportlab.graphics.renderbase import Renderer
from reportlab.graphics.shapes import Drawing
from reportlab.pdfgen.canvas import Canvas
from reportlab.platypus import Flowable

__version__: Final[str]

def draw(drawing: Drawing, canvas: Canvas, x: float, y: float, showBoundary=...) -> None: ...

class _PDFRenderer(Renderer):
    def __init__(self) -> None: ...
    def drawNode(self, node) -> None: ...
    def drawRect(self, rect) -> None: ...
    def drawImage(self, image) -> None: ...
    def drawLine(self, line) -> None: ...
    def drawCircle(self, circle) -> None: ...
    def drawPolyLine(self, polyline) -> None: ...
    def drawWedge(self, wedge) -> None: ...
    def drawEllipse(self, ellipse) -> None: ...
    def drawPolygon(self, polygon) -> None: ...
    def drawString(self, stringObj) -> None: ...
    def drawPath(self, path) -> None: ...
    def setStrokeColor(self, c) -> None: ...
    def setFillColor(self, c) -> None: ...
    def applyStateChanges(self, delta, newState) -> None: ...

class GraphicsFlowable(Flowable):
    drawing: Incomplete
    width: Incomplete
    height: Incomplete
    def __init__(self, drawing) -> None: ...
    def draw(self) -> None: ...

def drawToFile(d: Drawing, fn: str | IO[bytes], msg: str = "", showBoundary=..., autoSize: int = 1, **kwds) -> None: ...
def drawToString(d: Drawing, msg: str = "", showBoundary=..., autoSize: int = 1, **kwds) -> str: ...
def test(outDir: str = "pdfout", shout: bool = False) -> None: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import Any

from authlib.jose import BaseClaims

class ClientMetadataClaims(BaseClaims):
    def validate(self) -> None: ...
    def validate_redirect_uris(self) -> None: ...
    def validate_token_endpoint_auth_method(self) -> None: ...
    def validate_grant_types(self) -> None: ...
    def validate_response_types(self) -> None: ...
    def validate_client_name(self) -> None: ...
    def validate_client_uri(self) -> None: ...
    def validate_logo_uri(self) -> None: ...
    def validate_scope(self) -> None: ...
    def validate_contacts(self) -> None: ...
    def validate_tos_uri(self) -> None: ...
    def validate_policy_uri(self) -> None: ...
    def validate_jwks_uri(self) -> None: ...
    def validate_jwks(self) -> None: ...
    def validate_software_id(self) -> None: ...
    def validate_software_version(self) -> None: ...
    @classmethod
    def get_claims_options(cls, metadata: Mapping[str, Incomplete]) -> dict[str, Any]: ...  # dict values are key-dependent



import urllib.request
from _socket import _Address
from _typeshed import ReadableBuffer, SupportsRead, Unused
from collections.abc import Callable, Collection, Iterable, Mapping, MutableMapping
from http.client import HTTPResponse
from http.cookiejar import CookieJar
from subprocess import Popen
from typing import Any, AnyStr, TypeVar

has_certifi: bool
has_websockets: bool
_T = TypeVar("_T")

def load_plugins(name: str, suffix: str, namespace: dict[str, Any]) -> dict[str, type[Any]]: ...
def traverse_dict(dictn: Mapping[str, Any], keys: Collection[str], casesense: bool = True) -> Any: ...
def decode_base(value: str, digits: str) -> int: ...
def platform_name() -> str: ...
def get_subprocess_encoding() -> str: ...
def register_socks_protocols() -> None: ...
def handle_youtubedl_headers(headers: dict[str, Any]) -> dict[str, Any]: ...
def request_to_url(req: urllib.request.Request | str) -> str: ...
def sanitized_Request(
    url: str,
    data: ReadableBuffer | SupportsRead[bytes] | Iterable[bytes] | None = None,
    headers: MutableMapping[str, str] = {},
    origin_req_host: str | None = None,
    unverifiable: bool = False,
    method: str | None = None,
) -> urllib.request.Request: ...

class YoutubeDLHandler(urllib.request.AbstractHTTPHandler):
    def __init__(
        self,
        params: Mapping[str, Any],
        *args: Any,  # args passed to urllib.request.AbstractHTTPHandler.__init__().
        context: Any = None,
        source_address: _Address | None = None,
        debuglevel: int | None = None,
    ) -> None: ...

YoutubeDLHTTPSHandler = YoutubeDLHandler

class YoutubeDLCookieProcessor(urllib.request.HTTPCookieProcessor):
    def __init__(self, cookiejar: CookieJar | None = None) -> None: ...
    def http_response(self, request: urllib.request.Request, response: HTTPResponse) -> HTTPResponse: ...
    https_request: Callable[[urllib.request.HTTPCookieProcessor, urllib.request.Request], HTTPResponse]  # type: ignore[assignment]
    https_response = http_response

def make_HTTPS_handler(
    params: Mapping[str, Any], *, debuglevel: int | None = None, source_address: _Address | None = None
) -> YoutubeDLHTTPSHandler: ...
def process_communicate_or_kill(
    p: Popen[Any], *args: Any, **kwargs: Any  # args/kwargs passed to Popen.__init__().
) -> tuple[AnyStr, AnyStr]: ...
def encodeFilename(s: str, for_subprocess: Unused = False) -> bytes: ...
def decodeFilename(b: bytes, for_subprocess: Unused = False) -> str: ...
def decodeArgument(b: _T) -> _T: ...
def decodeOption(optval: AnyStr) -> str: ...
def error_to_compat_str(err: Any) -> str: ...  # Calls str(err).



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["k_clique_communities"]

@_dispatchable
def k_clique_communities(G: Graph[_Node], k: int, cliques=None) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "projected_graph",
    "weighted_projected_graph",
    "collaboration_weighted_projected_graph",
    "overlap_weighted_projected_graph",
    "generic_weighted_projected_graph",
]

@_dispatchable
def projected_graph(B: Graph[_Node], nodes: Iterable[Incomplete], multigraph: bool = False): ...
@_dispatchable
def weighted_projected_graph(B: Graph[_Node], nodes: Iterable[Incomplete], ratio: bool = False): ...
@_dispatchable
def collaboration_weighted_projected_graph(B: Graph[_Node], nodes: Iterable[Incomplete]): ...
@_dispatchable
def overlap_weighted_projected_graph(B: Graph[_Node], nodes: Iterable[Incomplete], jaccard: bool = True): ...
@_dispatchable
def generic_weighted_projected_graph(
    B: Graph[_Node], nodes: Iterable[Incomplete], weight_function: Callable[..., Incomplete] | None = None
): ...



from typing import List
import warnings

from lsprotocol import types

from .workspace import Workspace
from .text_document import TextDocument
from .position_codec import PositionCodec

# For backwards compatibility
Document = TextDocument


def utf16_unit_offset(chars: str):
    warnings.warn(
        "'utf16_unit_offset' has been deprecated, instead use "
        "'PositionCodec.utf16_unit_offset' via 'workspace.position_codec' "
        "or 'text_document.position_codec'",
        DeprecationWarning,
        stacklevel=2,
    )
    _codec = PositionCodec()
    return _codec.utf16_unit_offset(chars)


def utf16_num_units(chars: str):
    warnings.warn(
        "'utf16_num_units' has been deprecated, instead use "
        "'PositionCodec.client_num_units' via 'workspace.position_codec' "
        "or 'text_document.position_codec'",
        DeprecationWarning,
        stacklevel=2,
    )
    _codec = PositionCodec()
    return _codec.client_num_units(chars)


def position_from_utf16(lines: List[str], position: types.Position):
    warnings.warn(
        "'position_from_utf16' has been deprecated, instead use "
        "'PositionCodec.position_from_client_units' via "
        "'workspace.position_codec' or 'text_document.position_codec'",
        DeprecationWarning,
        stacklevel=2,
    )
    _codec = PositionCodec()
    return _codec.position_from_client_units(lines, position)


def position_to_utf16(lines: List[str], position: types.Position):
    warnings.warn(
        "'position_to_utf16' has been deprecated, instead use "
        "'PositionCodec.position_to_client_units' via "
        "'workspace.position_codec' or 'text_document.position_codec'",
        DeprecationWarning,
        stacklevel=2,
    )
    _codec = PositionCodec()
    return _codec.position_to_client_units(lines, position)


def range_from_utf16(lines: List[str], range: types.Range):
    warnings.warn(
        "'range_from_utf16' has been deprecated, instead use "
        "'PositionCodec.range_from_client_units' via "
        "'workspace.position_codec' or 'text_document.position_codec'",
        DeprecationWarning,
        stacklevel=2,
    )
    _codec = PositionCodec()
    return _codec.range_from_client_units(lines, range)


def range_to_utf16(lines: List[str], range: types.Range):
    warnings.warn(
        "'range_to_utf16' has been deprecated, instead use "
        "'PositionCodec.range_to_client_units' via 'workspace.position_codec' "
        "or 'text_document.position_codec'",
        DeprecationWarning,
        stacklevel=2,
    )
    _codec = PositionCodec()
    return _codec.range_to_client_units(lines, range)


__all__ = (
    "Workspace",
    "TextDocument",
    "PositionCodec",
    "Document",
    "utf16_unit_offset",
    "utf16_num_units",
    "position_from_utf16",
    "position_to_utf16",
    "range_from_utf16",
    "range_to_utf16",
)



import random
import sys
from types import ModuleType
from typing_extensions import Self, TypeAlias

import numpy
from networkx.classes.graph import Graph, _Node

__all__ = [
    "flatten",
    "make_list_of_ints",
    "dict_to_numpy_array",
    "arbitrary_element",
    "pairwise",
    "groups",
    "create_random_state",
    "create_py_random_state",
    "PythonRandomInterface",
    "PythonRandomViaNumpyBits",
    "nodes_equal",
    "edges_equal",
    "graphs_equal",
    "_clear_cache",
]

_RandomNumberGenerator: TypeAlias = (
    ModuleType | random.Random | numpy.random.RandomState | numpy.random.Generator | PythonRandomInterface
)
_RandomState: TypeAlias = int | _RandomNumberGenerator | None

def flatten(obj, result=None): ...
def make_list_of_ints(sequence): ...
def dict_to_numpy_array(d, mapping=None): ...
def arbitrary_element(iterable): ...
def pairwise(iterable, cyclic: bool = False): ...
def groups(many_to_one): ...
def create_random_state(random_state=None): ...

class PythonRandomViaNumpyBits(random.Random):
    def __init__(self, rng: numpy.random.Generator | None = None) -> None: ...
    if sys.version_info < (3, 10):
        # this is a workaround for pyright correctly flagging an inconsistent inherited constructor, see #14624
        def __new__(cls, rng: numpy.random.Generator | None = None) -> Self: ...

    def getrandbits(self, k: int) -> int: ...

class PythonRandomInterface:
    def __init__(self, rng=None) -> None: ...
    def random(self): ...
    def uniform(self, a, b): ...
    def randrange(self, a, b=None): ...
    def choice(self, seq): ...
    def gauss(self, mu, sigma): ...
    def shuffle(self, seq): ...
    def sample(self, seq, k): ...
    def randint(self, a, b): ...
    def expovariate(self, scale): ...
    def paretovariate(self, shape): ...

def create_py_random_state(random_state: _RandomState = None): ...
def nodes_equal(nodes1, nodes2) -> bool: ...
def edges_equal(edges1, edges2) -> bool: ...
def graphs_equal(graph1, graph2) -> bool: ...
def _clear_cache(G: Graph[_Node]) -> None: ...



from typing import Literal

__version__: str
__version_info__: tuple[int, int, int]
system: str

def user_cache_dir(
    appname: str | None = None, appauthor: Literal[False] | str | None = None, version: str | None = None, opinion: bool = True
) -> str: ...



from collections.abc import Callable

class Hookable:
    def __init__(self) -> None: ...
    def register_hook(self, hook_type: str, hook: Callable[..., None]) -> None: ...
    def execute_hook(self, hook_type: str, *args, **kwargs) -> None: ...

def hooked(func=None, before: str | None = None, after: str | None = None): ...



from _typeshed import Incomplete

from httplib2 import Response

class HttpLib2Error(Exception): ...

class HttpLib2ErrorWithResponse(HttpLib2Error):
    response: Response | dict[str, Incomplete] | None
    content: str | bytes | None
    def __init__(
        self, desc: str | None, response: Response | dict[str, Incomplete] | None, content: str | bytes | None
    ) -> None: ...

class RedirectMissingLocation(HttpLib2ErrorWithResponse): ...
class RedirectLimit(HttpLib2ErrorWithResponse): ...
class FailedToDecompressContent(HttpLib2ErrorWithResponse): ...
class UnimplementedDigestAuthOptionError(HttpLib2ErrorWithResponse): ...
class UnimplementedHmacDigestAuthOptionError(HttpLib2ErrorWithResponse): ...
class MalformedHeader(HttpLib2Error): ...
class RelativeURIError(HttpLib2Error): ...
class ServerNotFoundError(HttpLib2Error): ...
class ProxiesUnavailableError(HttpLib2Error): ...



import socket
from _socket import _Address
from collections.abc import Mapping
from typing import Final, NamedTuple, SupportsIndex

SOCKS4_VERSION: Final = 4
SOCKS4_REPLY_VERSION: Final = 0x00
SOCKS4_DEFAULT_DSTIP: Final[bytes]
SOCKS5_VERSION: Final = 5
SOCKS5_USER_AUTH_VERSION: Final = 0x01
SOCKS5_USER_AUTH_SUCCESS: Final = 0x00

class Socks4Command:
    CMD_CONNECT: int
    CMD_BIND: int

class Socks5Command(Socks4Command):
    CMD_UDP_ASSOCIATE: int

class Socks5Auth:
    AUTH_NONE: int
    AUTH_GSSAPI: int
    AUTH_USER_PASS: int
    AUTH_NO_ACCEPTABLE: int

class Socks5AddressType:
    ATYP_IPV4: int
    ATYP_DOMAINNAME: int
    ATYP_IPV6: int

class ProxyError(OSError):
    ERR_SUCCESS: int
    def __init__(self, code: int | None = None, msg: str | None = None) -> None: ...

class InvalidVersionError(ProxyError):
    def __init__(self, expected_version: int, got_version: int) -> None: ...

class Socks4Error(ProxyError):
    ERR_SUCCESS: int
    CODES: Mapping[int, str]

class Socks5Error(ProxyError):
    ERR_GENERAL_FAILURE: int
    CODES: Mapping[int, str]

class ProxyType:
    SOCKS4: int
    SOCKS4A: int
    SOCKS5: int

class Proxy(NamedTuple):
    type: ProxyType
    host: str
    port: int
    username: str
    password: str
    remote_dns: bool

class sockssocket(socket.socket):
    def __init__(
        self, family: int = -1, type: int = -1, proto: int = -1, fileno: SupportsIndex | bytes | None = None
    ) -> None: ...
    def setproxy(
        self,
        proxytype: ProxyType,
        addr: str,
        port: int,
        rdns: bool = True,
        username: str | None = None,
        password: str | None = None,
    ) -> None: ...
    def recvall(self, cnt: int) -> bytes: ...
    def connect(self, address: _Address) -> None: ...
    def connect_ex(self, address: _Address) -> int: ...



import re
from _typeshed import Incomplete
from typing import ClassVar

from pony.orm import dbapiprovider, sqltranslation
from pony.orm.dbapiprovider import DBAPIProvider
from pony.orm.dbschema import Column, DBObject, DBSchema, Table
from pony.orm.sqlbuilding import SQLBuilder

NoneType: type[None]

class OraTable(Table):
    def get_objects_to_create(table, created_tables=None): ...

class OraSequence(DBObject):
    typename: ClassVar[str]
    def __init__(sequence, table, name=None) -> None: ...
    def exists(sequence, provider, connection, case_sensitive: bool = True): ...
    def get_create_command(sequence): ...

trigger_template: str

class OraTrigger(DBObject):
    typename: ClassVar[str]
    def __init__(trigger, table, column, sequence) -> None: ...
    def exists(trigger, provider, connection, case_sensitive: bool = True): ...
    def get_create_command(trigger): ...

class OraColumn(Column):
    auto_template: ClassVar[None]  # type: ignore[assignment]

class OraSchema(DBSchema):
    dialect: ClassVar[str]
    table_class: ClassVar[type[OraTable]]
    column_class: ClassVar[type[OraColumn]]

class OraNoneMonad(sqltranslation.NoneMonad):
    def __init__(monad, value=None) -> None: ...

class OraConstMonad(sqltranslation.ConstMonad):
    @staticmethod
    def new(value): ...

class OraTranslator(sqltranslation.SQLTranslator):
    dialect: ClassVar[str]
    NoneMonad = OraNoneMonad
    ConstMonad = OraConstMonad

class OraBuilder(SQLBuilder):
    dialect: ClassVar[str]
    def INSERT(builder, table_name, columns, values, returning=None): ...
    def SELECT_FOR_UPDATE(builder, nowait, skip_locked, *sections): ...
    def SELECT(builder, *sections): ...
    def ROWID(builder, *expr_list): ...
    def LIMIT(builder, limit, offset=None) -> None: ...
    def TO_REAL(builder, expr): ...
    def TO_STR(builder, expr): ...
    def DATE(builder, expr): ...
    def RANDOM(builder): ...
    def MOD(builder, a, b): ...
    def DATE_ADD(builder, expr, delta): ...
    def DATE_SUB(builder, expr, delta): ...
    def DATE_DIFF(builder, expr1, expr2): ...
    def DATETIME_ADD(builder, expr, delta): ...
    def DATETIME_SUB(builder, expr, delta): ...
    def DATETIME_DIFF(builder, expr1, expr2): ...
    def build_json_path(builder, path): ...
    def JSON_QUERY(builder, expr, path): ...
    json_value_type_mapping: Incomplete
    def JSON_VALUE(builder, expr, path, type): ...
    def JSON_NONZERO(builder, expr): ...
    def JSON_CONTAINS(builder, expr, path, key): ...
    def JSON_ARRAY_LENGTH(builder, value) -> None: ...
    def GROUP_CONCAT(builder, distinct, expr, sep=None): ...

json_item_re: re.Pattern[str]

class OraBoolConverter(dbapiprovider.BoolConverter):
    def py2sql(converter, val): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class OraStrConverter(dbapiprovider.StrConverter):
    def validate(converter, val, obj=None): ...
    def sql2py(converter, val): ...
    def sql_type(converter): ...

class OraIntConverter(dbapiprovider.IntConverter):
    signed_types: Incomplete
    unsigned_types: Incomplete
    def init(self, kwargs) -> None: ...

class OraRealConverter(dbapiprovider.RealConverter):
    def sql_type(converter): ...

class OraDecimalConverter(dbapiprovider.DecimalConverter):
    def sql_type(converter): ...

class OraBlobConverter(dbapiprovider.BlobConverter):
    def sql2py(converter, val): ...

class OraDateConverter(dbapiprovider.DateConverter):
    def sql2py(converter, val): ...

class OraTimeConverter(dbapiprovider.TimeConverter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...
    def sql2py(converter, val): ...
    def py2sql(converter, val): ...

class OraTimedeltaConverter(dbapiprovider.TimedeltaConverter):
    def __init__(converter, provider, py_type, attr=None) -> None: ...

class OraDatetimeConverter(dbapiprovider.DatetimeConverter): ...

class OraUuidConverter(dbapiprovider.UuidConverter):
    def sql_type(converter): ...

class OraJsonConverter(dbapiprovider.JsonConverter):
    json_kwargs: Incomplete
    optimistic: bool
    def sql2py(converter, dbval): ...
    def sql_type(converter): ...

class OraProvider(DBAPIProvider):
    dialect: ClassVar[str]
    varchar_default_max_len: ClassVar[int]
    dbschema_cls: ClassVar[type[OraSchema]]
    translator_cls: ClassVar[type[OraTranslator]]
    sqlbuilder_cls: ClassVar[type[OraBuilder]]
    converter_classes: Incomplete
    def inspect_connection(provider, connection) -> None: ...
    def should_reconnect(provider, exc): ...
    def normalize_name(provider, name): ...
    def normalize_vars(provider, vars, vartypes) -> None: ...
    def set_transaction_mode(provider, connection, cache) -> None: ...
    def execute(provider, cursor, sql, arguments=None, returning_id: bool = False): ...
    def get_pool(provider, *args, **kwargs): ...
    def table_exists(provider, connection, table_name, case_sensitive: bool = True): ...
    def index_exists(provider, connection, table_name, index_name, case_sensitive: bool = True): ...
    def fk_exists(provider, connection, table_name, fk_name, case_sensitive: bool = True): ...
    def table_has_data(provider, connection, table_name): ...
    def drop_table(provider, connection, table_name) -> None: ...

provider_cls = OraProvider

def to_int_or_decimal(val): ...
def to_decimal(val): ...
def output_type_handler(cursor, name, defaultType, size, precision, scale): ...

class OraPool:
    forked_pools: Incomplete
    def __init__(pool, **kwargs) -> None: ...
    def connect(pool): ...
    def release(pool, con) -> None: ...
    def drop(pool, con) -> None: ...
    def disconnect(pool) -> None: ...

def get_inputsize(arg): ...
def set_input_sizes(cursor, arguments) -> None: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from typing import overload
from typing_extensions import deprecated

from authlib.oauth2.rfc6749 import ClientMixin

class OAuth2Payload:
    @property
    def data(self) -> dict[str, str]: ...
    @property
    def datalist(self) -> dict[str, list[Incomplete]]: ...
    @property
    def client_id(self) -> str: ...
    @property
    def response_type(self) -> str: ...
    @property
    def grant_type(self) -> str: ...
    @property
    def redirect_uri(self) -> str: ...
    @property
    def scope(self) -> str: ...
    @property
    def state(self) -> str | None: ...

class BasicOAuth2Payload(OAuth2Payload):
    def __init__(self, payload: dict[str, str]) -> None: ...
    @property
    def data(self) -> dict[str, str]: ...
    @property
    def datalist(self) -> dict[str, list[Incomplete]]: ...

class OAuth2Request(OAuth2Payload):
    method: str
    uri: str
    headers: Mapping[str, str] | None
    payload: OAuth2Payload | None
    client: ClientMixin | None
    auth_method: str | None
    user: Incomplete | None
    authorization_code: Incomplete | None
    refresh_token: Incomplete | None
    credential: Incomplete | None
    @overload
    def __init__(self, method: str, uri: str, body: None = None, headers: Mapping[str, str] | None = None) -> None: ...
    @overload
    @deprecated("The `body` parameter in OAuth2Request is deprecated. Use the payload system instead.")
    def __init__(self, method: str, uri: str, body, headers: Mapping[str, str] | None = None) -> None: ...
    @property
    def args(self) -> dict[str, str | None]: ...
    @property
    def form(self) -> dict[str, str]: ...
    @property
    @deprecated("'request.data' is deprecated in favor of 'request.payload.data'")
    def data(self) -> dict[str, str]: ...
    @property
    @deprecated("'request.datalist' is deprecated in favor of 'request.payload.datalist'")
    def datalist(self) -> dict[str, list[Incomplete]]: ...
    @property
    @deprecated("'request.client_id' is deprecated in favor of 'request.payload.client_id'")
    def client_id(self) -> str: ...
    @property
    @deprecated("'request.response_type' is deprecated in favor of 'request.payload.response_type'")
    def response_type(self) -> str: ...
    @property
    @deprecated("'request.grant_type' is deprecated in favor of 'request.payload.grant_type'")
    def grant_type(self) -> str: ...
    @property
    @deprecated("'request.redirect_uri' is deprecated in favor of 'request.payload.redirect_uri'")
    def redirect_uri(self) -> str: ...
    @property
    @deprecated("'request.scope' is deprecated in favor of 'request.payload.scope'")
    def scope(self) -> str: ...
    @property
    @deprecated("'request.state' is deprecated in favor of 'request.payload.state'")
    def state(self) -> str | None: ...
    @property
    @deprecated("'request.body' is deprecated. Use the payload system instead.")
    def body(self): ...

class JsonPayload:
    @property
    def data(self): ...

class JsonRequest:
    method: str
    uri: str
    payload: JsonPayload | None
    headers: Mapping[str, str]
    def __init__(self, method: str, uri: str, headers: Mapping[str, str] | None = None) -> None: ...
    @property
    @deprecated("'request.data' is deprecated in favor of 'request.payload.data'")
    def data(self): ...



from _typeshed import Incomplete
from collections.abc import Generator, Iterator

class UnionFind:
    parents: Incomplete
    weights: Incomplete
    def __init__(self, elements=None) -> None: ...
    def __getitem__(self, object): ...
    def __iter__(self) -> Iterator[Incomplete]: ...
    def to_sets(self) -> Generator[Incomplete, Incomplete, None]: ...
    def union(self, *objects): ...



import datetime

def to_datetime(jdc: float) -> datetime.datetime: ...
def from_datetime(dt: datetime.datetime) -> float: ...
def to_gregorian(jdc: float) -> tuple[int, int, int]: ...
def from_gregorian(year: int, month: int, day: int) -> float: ...
def to_julian(jdc: float) -> tuple[int, int, int]: ...
def from_julian(year: int, month: int, day: int) -> float: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["efficiency", "local_efficiency", "global_efficiency"]

@_dispatchable
def efficiency(G: Graph[_Node], u: _Node, v: _Node) -> float: ...
@_dispatchable
def global_efficiency(G: Graph[_Node]) -> float: ...
@_dispatchable
def local_efficiency(G: Graph[_Node]) -> float: ...



from _typeshed import Incomplete
from typing import Any

from google.cloud.ndb import _options

class PropertyOrder:
    name: Any
    reverse: Any
    def __init__(self, name, reverse: bool = ...) -> None: ...
    def __neg__(self): ...

class RepeatedStructuredPropertyPredicate:
    name: Any
    match_keys: Any
    match_values: Any
    def __init__(self, name, match_keys, entity_pb) -> None: ...
    def __call__(self, entity_pb): ...

class ParameterizedThing:
    def __eq__(self, other): ...
    def __ne__(self, other): ...

class Parameter(ParameterizedThing):
    def __init__(self, key) -> None: ...
    def __eq__(self, other): ...
    @property
    def key(self): ...
    def resolve(self, bindings, used): ...

class ParameterizedFunction(ParameterizedThing):
    func: Any
    values: Any
    def __init__(self, func, values) -> None: ...
    def __eq__(self, other): ...
    def is_parameterized(self): ...
    def resolve(self, bindings, used): ...

class Node:
    def __new__(cls): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __le__(self, unused_other): ...
    def __lt__(self, unused_other): ...
    def __ge__(self, unused_other): ...
    def __gt__(self, unused_other): ...
    def resolve(self, bindings, used): ...

class FalseNode(Node):
    def __eq__(self, other): ...

class ParameterNode(Node):
    def __new__(cls, prop, op, param): ...
    def __getnewargs__(self): ...
    def __eq__(self, other): ...
    def resolve(self, bindings, used): ...

class FilterNode(Node):
    def __new__(cls, name, opsymbol, value, server_op: bool = False): ...
    def __getnewargs__(self): ...
    def __eq__(self, other): ...

class PostFilterNode(Node):
    def __new__(cls, predicate): ...
    def __getnewargs__(self): ...
    def __eq__(self, other): ...

class _BooleanClauses:
    name: Any
    combine_or: Any
    or_parts: Any
    def __init__(self, name, combine_or) -> None: ...
    def add_node(self, node) -> None: ...

class ConjunctionNode(Node):
    def __new__(cls, *nodes): ...
    def __getnewargs__(self): ...
    def __iter__(self): ...
    def __eq__(self, other): ...
    def resolve(self, bindings, used): ...

class DisjunctionNode(Node):
    def __new__(cls, *nodes): ...
    def __getnewargs__(self): ...
    def __iter__(self): ...
    def __eq__(self, other): ...
    def resolve(self, bindings, used): ...

AND = ConjunctionNode
OR = DisjunctionNode

class QueryOptions(_options.ReadOptions):
    project: Any
    namespace: Any
    database: str | None
    def __init__(self, config: Incomplete | None = ..., context: Incomplete | None = ..., **kwargs) -> None: ...

class Query:
    default_options: Any
    kind: Any
    ancestor: Any
    filters: Any
    order_by: Any
    project: Any
    namespace: Any
    limit: Any
    offset: Any
    keys_only: Any
    projection: Any
    distinct_on: Any
    database: str | None
    def __init__(
        self,
        kind: Incomplete | None = ...,
        filters: Incomplete | None = ...,
        ancestor: Incomplete | None = ...,
        order_by: Incomplete | None = ...,
        orders: Incomplete | None = ...,
        project: Incomplete | None = ...,
        app: Incomplete | None = ...,
        namespace: Incomplete | None = ...,
        projection: Incomplete | None = ...,
        distinct_on: Incomplete | None = ...,
        group_by: Incomplete | None = ...,
        limit: Incomplete | None = ...,
        offset: Incomplete | None = ...,
        keys_only: Incomplete | None = ...,
        default_options: Incomplete | None = ...,
    ) -> None: ...
    @property
    def is_distinct(self): ...
    def filter(self, *filters): ...
    def order(self, *props): ...
    def analyze(self): ...
    def bind(self, *positional, **keyword): ...
    def fetch(self, limit: Incomplete | None = ..., **kwargs): ...
    def fetch_async(self, limit: Incomplete | None = ..., **kwargs): ...
    def run_to_queue(self, queue, conn, options: Incomplete | None = ..., dsquery: Incomplete | None = ...) -> None: ...
    def iter(self, **kwargs): ...
    __iter__: Any
    def map(self, callback, **kwargs): ...
    def map_async(self, callback, **kwargs) -> None: ...
    def get(self, **kwargs): ...
    def get_async(self, **kwargs) -> None: ...
    def count(self, limit: Incomplete | None = ..., **kwargs): ...
    def count_async(self, limit: Incomplete | None = ..., **kwargs): ...
    def fetch_page(self, page_size, **kwargs): ...
    def fetch_page_async(self, page_size, **kwargs) -> None: ...

def gql(query_string: str, *args: Any, **kwds: Any) -> Query: ...



import ast

from flake8_simplify.utils import If

def get_sim102(node: ast.If) -> list[tuple[int, int, str]]: ...
def get_sim103(node: ast.If) -> list[tuple[int, int, str]]: ...
def get_sim108(node: If) -> list[tuple[int, int, str]]: ...
def get_sim114(node: ast.If) -> list[tuple[int, int, str]]: ...
def get_sim116(node: ast.If) -> list[tuple[int, int, str]]: ...
def get_sim908(node: ast.If) -> list[tuple[int, int, str]]: ...
def get_sim401(node: ast.If) -> list[tuple[int, int, str]]: ...



from _typeshed import Incomplete
from abc import ABC, abstractmethod

from .syntax import PDFObject

class Action(ABC):
    next: PDFObject | str | None
    def __init__(self, next_action: PDFObject | str | None = None) -> None: ...
    @abstractmethod
    def serialize(self) -> str: ...

class URIAction(Action):
    uri: str
    def __init__(self, uri: str, next_action: PDFObject | str | None = None) -> None: ...
    def serialize(self) -> str: ...

class NamedAction(Action):
    action_name: str
    def __init__(self, action_name: str, next_action: PDFObject | str | None = None) -> None: ...
    def serialize(self) -> str: ...

class GoToAction(Action):
    dest: Incomplete
    def __init__(self, dest, next_action: PDFObject | str | None = None) -> None: ...
    def serialize(self) -> str: ...

class GoToRemoteAction(Action):
    file: str
    dest: Incomplete
    def __init__(self, file: str, dest, next_action: PDFObject | str | None = None) -> None: ...
    def serialize(self) -> str: ...

class LaunchAction(Action):
    file: str
    def __init__(self, file: str, next_action: PDFObject | str | None = None) -> None: ...
    def serialize(self) -> str: ...



from .errors import InsufficientScopeError as InsufficientScopeError, InvalidTokenError as InvalidTokenError
from .parameters import add_bearer_token as add_bearer_token
from .token import BearerTokenGenerator as BearerTokenGenerator
from .validator import BearerTokenValidator as BearerTokenValidator

__all__ = [
    "InvalidTokenError",
    "InsufficientScopeError",
    "add_bearer_token",
    "BearerToken",
    "BearerTokenGenerator",
    "BearerTokenValidator",
]

BearerToken = BearerTokenGenerator



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CostGraphDef(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Node(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        @typing.final
        class InputInfo(google.protobuf.message.Message):
            """Inputs of this node. They must be executed before this node can be
            executed. An input is a particular output of another node, specified
            by the node id and the output index.
            """

            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            PRECEDING_NODE_FIELD_NUMBER: builtins.int
            PRECEDING_PORT_FIELD_NUMBER: builtins.int
            preceding_node: builtins.int
            preceding_port: builtins.int
            def __init__(
                self, *, preceding_node: builtins.int | None = ..., preceding_port: builtins.int | None = ...
            ) -> None: ...
            def ClearField(
                self, field_name: typing.Literal["preceding_node", b"preceding_node", "preceding_port", b"preceding_port"]
            ) -> None: ...

        @typing.final
        class OutputInfo(google.protobuf.message.Message):
            """Outputs of this node."""

            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            SIZE_FIELD_NUMBER: builtins.int
            ALIAS_INPUT_PORT_FIELD_NUMBER: builtins.int
            SHAPE_FIELD_NUMBER: builtins.int
            DTYPE_FIELD_NUMBER: builtins.int
            size: builtins.int
            alias_input_port: builtins.int
            """If >= 0, the output is an alias of an input. Note that an alias input
            may itself be an alias. The algorithm will therefore need to follow
            those pointers.
            """
            dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
            @property
            def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
            def __init__(
                self,
                *,
                size: builtins.int | None = ...,
                alias_input_port: builtins.int | None = ...,
                shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
                dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
            ) -> None: ...
            def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
            def ClearField(
                self,
                field_name: typing.Literal[
                    "alias_input_port", b"alias_input_port", "dtype", b"dtype", "shape", b"shape", "size", b"size"
                ],
            ) -> None: ...

        NAME_FIELD_NUMBER: builtins.int
        DEVICE_FIELD_NUMBER: builtins.int
        ID_FIELD_NUMBER: builtins.int
        INPUT_INFO_FIELD_NUMBER: builtins.int
        OUTPUT_INFO_FIELD_NUMBER: builtins.int
        TEMPORARY_MEMORY_SIZE_FIELD_NUMBER: builtins.int
        PERSISTENT_MEMORY_SIZE_FIELD_NUMBER: builtins.int
        HOST_TEMP_MEMORY_SIZE_FIELD_NUMBER: builtins.int
        DEVICE_TEMP_MEMORY_SIZE_FIELD_NUMBER: builtins.int
        DEVICE_PERSISTENT_MEMORY_SIZE_FIELD_NUMBER: builtins.int
        COMPUTE_COST_FIELD_NUMBER: builtins.int
        COMPUTE_TIME_FIELD_NUMBER: builtins.int
        MEMORY_TIME_FIELD_NUMBER: builtins.int
        IS_FINAL_FIELD_NUMBER: builtins.int
        CONTROL_INPUT_FIELD_NUMBER: builtins.int
        INACCURATE_FIELD_NUMBER: builtins.int
        name: builtins.str
        """The name of the node. Names are globally unique."""
        device: builtins.str
        """The device of the node. Can be empty if the node is mapped to the
        default partition or partitioning hasn't been run yet.
        """
        id: builtins.int
        """The id of the node. Node ids are only unique inside a partition."""
        temporary_memory_size: builtins.int
        """Temporary memory used by this node."""
        persistent_memory_size: builtins.int
        """Persistent memory used by this node."""
        host_temp_memory_size: builtins.int
        device_temp_memory_size: builtins.int
        device_persistent_memory_size: builtins.int
        compute_cost: builtins.int
        """Estimate of the computational cost of this node, in microseconds."""
        compute_time: builtins.int
        """Analytical estimate of the computational cost of this node, in
        microseconds.
        """
        memory_time: builtins.int
        """Analytical estimate of the memory access cost of this node, in
        microseconds.
        """
        is_final: builtins.bool
        """If true, the output is permanent: it can't be discarded, because this
        node is part of the "final output". Nodes may depend on final nodes.
        """
        inaccurate: builtins.bool
        """Are the costs inaccurate?"""
        @property
        def input_info(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CostGraphDef.Node.InputInfo]: ...
        @property
        def output_info(
            self,
        ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CostGraphDef.Node.OutputInfo]: ...
        @property
        def control_input(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """Ids of the control inputs for this node."""

        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            device: builtins.str | None = ...,
            id: builtins.int | None = ...,
            input_info: collections.abc.Iterable[global___CostGraphDef.Node.InputInfo] | None = ...,
            output_info: collections.abc.Iterable[global___CostGraphDef.Node.OutputInfo] | None = ...,
            temporary_memory_size: builtins.int | None = ...,
            persistent_memory_size: builtins.int | None = ...,
            host_temp_memory_size: builtins.int | None = ...,
            device_temp_memory_size: builtins.int | None = ...,
            device_persistent_memory_size: builtins.int | None = ...,
            compute_cost: builtins.int | None = ...,
            compute_time: builtins.int | None = ...,
            memory_time: builtins.int | None = ...,
            is_final: builtins.bool | None = ...,
            control_input: collections.abc.Iterable[builtins.int] | None = ...,
            inaccurate: builtins.bool | None = ...,
        ) -> None: ...
        def ClearField(
            self,
            field_name: typing.Literal[
                "compute_cost",
                b"compute_cost",
                "compute_time",
                b"compute_time",
                "control_input",
                b"control_input",
                "device",
                b"device",
                "device_persistent_memory_size",
                b"device_persistent_memory_size",
                "device_temp_memory_size",
                b"device_temp_memory_size",
                "host_temp_memory_size",
                b"host_temp_memory_size",
                "id",
                b"id",
                "inaccurate",
                b"inaccurate",
                "input_info",
                b"input_info",
                "is_final",
                b"is_final",
                "memory_time",
                b"memory_time",
                "name",
                b"name",
                "output_info",
                b"output_info",
                "persistent_memory_size",
                b"persistent_memory_size",
                "temporary_memory_size",
                b"temporary_memory_size",
            ],
        ) -> None: ...

    @typing.final
    class AggregatedCost(google.protobuf.message.Message):
        """Total cost of this graph, typically used for balancing decisions."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        COST_FIELD_NUMBER: builtins.int
        DIMENSION_FIELD_NUMBER: builtins.int
        cost: builtins.float
        """Aggregated cost value."""
        dimension: builtins.str
        """Aggregated cost dimension (e.g. 'memory', 'compute', 'network')."""
        def __init__(self, *, cost: builtins.float | None = ..., dimension: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["cost", b"cost", "dimension", b"dimension"]) -> None: ...

    NODE_FIELD_NUMBER: builtins.int
    COST_FIELD_NUMBER: builtins.int
    @property
    def node(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CostGraphDef.Node]: ...
    @property
    def cost(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CostGraphDef.AggregatedCost]: ...
    def __init__(
        self,
        *,
        node: collections.abc.Iterable[global___CostGraphDef.Node] | None = ...,
        cost: collections.abc.Iterable[global___CostGraphDef.AggregatedCost] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cost", b"cost", "node", b"node"]) -> None: ...

global___CostGraphDef = CostGraphDef



from collections.abc import Callable
from datetime import date, datetime, time
from itertools import repeat
from re import Pattern
from typing import Any, overload
from typing_extensions import TypeAlias

_CustomTagsParser: TypeAlias = Callable[[str, int, dict[str, Any], dict[str, Any]], object]

ATTRIBUTELISTPATTERN: Pattern[str]

def cast_date_time(value: str) -> datetime: ...
@overload
def format_date_time(value: time, *, timespec: str = ...) -> str: ...
@overload
def format_date_time(value: datetime, *, sep: str = ..., timespec: str = ...) -> str: ...
@overload
def format_date_time(value: date) -> str: ...

class ParseError(Exception):
    lineno: int
    line: str
    def __init__(self, lineno: int, line: str) -> None: ...

def parse(content: str, strict: bool = False, custom_tags_parser: _CustomTagsParser | None = None) -> dict[str, Any]: ...
def string_to_lines(string: str) -> list[str]: ...
def remove_quotes_parser(*attrs: repeat[Callable[[str], str]]) -> dict[repeat[Callable[[str], str]], Callable[[str], str]]: ...
def remove_quotes(string: str) -> str: ...
def normalize_attribute(attribute: str) -> str: ...



import grpc_channelz.v1.channelz_pb2 as _channelz_pb2
import grpc_channelz.v1.channelz_pb2_grpc as _channelz_pb2_grpc
from grpc import ServicerContext

class ChannelzServicer(_channelz_pb2_grpc.ChannelzServicer):
    @staticmethod
    def GetTopChannels(
        request: _channelz_pb2.GetTopChannelsRequest, context: ServicerContext
    ) -> _channelz_pb2.GetTopChannelsResponse: ...
    @staticmethod
    def GetServers(request: _channelz_pb2.GetServersRequest, context: ServicerContext) -> _channelz_pb2.GetServersResponse: ...
    @staticmethod
    def GetServer(request: _channelz_pb2.GetServerRequest, context: ServicerContext) -> _channelz_pb2.GetServerResponse: ...
    @staticmethod
    def GetServerSockets(
        request: _channelz_pb2.GetServerSocketsRequest, context: ServicerContext
    ) -> _channelz_pb2.GetServerSocketsResponse: ...
    @staticmethod
    def GetChannel(request: _channelz_pb2.GetChannelRequest, context: ServicerContext) -> _channelz_pb2.GetChannelResponse: ...
    @staticmethod
    def GetSubchannel(
        request: _channelz_pb2.GetSubchannelRequest, context: ServicerContext
    ) -> _channelz_pb2.GetSubchannelResponse: ...
    @staticmethod
    def GetSocket(request: _channelz_pb2.GetSocketRequest, context: ServicerContext) -> _channelz_pb2.GetSocketResponse: ...



from typing import Final
from xml.sax.expatreader import ExpatParser as _ExpatParser, _BoolType

__origin__: Final = "xml.sax.expatreader"

class DefusedExpatParser(_ExpatParser):
    forbid_dtd: bool
    forbid_entities: bool
    forbid_external: bool
    def __init__(
        self,
        namespaceHandling: _BoolType = 0,
        bufsize: int = 65516,
        forbid_dtd: bool = False,
        forbid_entities: bool = True,
        forbid_external: bool = True,
    ) -> None: ...
    def defused_start_doctype_decl(self, name: str, sysid: str | None, pubid: str | None, has_internal_subset: bool) -> None: ...
    def defused_entity_decl(
        self,
        name: str,
        is_parameter_entity: bool,
        value: str | None,
        base: str | None,
        sysid: str,
        pubid: str | None,
        notation_name: str | None,
    ) -> None: ...
    def defused_unparsed_entity_decl(
        self, name: str, base: str | None, sysid: str, pubid: str | None, notation_name: str
    ) -> None: ...
    def defused_external_entity_ref_handler(
        self, context: str, base: str | None, sysid: str | None, pubid: str | None
    ) -> None: ...
    def reset(self) -> None: ...

def create_parser(
    namespaceHandling: _BoolType = 0,
    bufsize: int = 65516,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> DefusedExpatParser: ...



from turtle import Turtle, dot

Turtle().dot()
Turtle().dot(10)
Turtle().dot(size=10)
Turtle().dot((0, 0, 0))
Turtle().dot(size=(0, 0, 0))
Turtle().dot("blue")
Turtle().dot("")
Turtle().dot(size="blue")
Turtle().dot(20, "blue")
Turtle().dot(20, "blue")
Turtle().dot(20, (0, 0, 0))
Turtle().dot(20, 0, 0, 0)

Turtle().dot(size=10, color="blue")  # type: ignore
Turtle().dot(10, color="blue")  # type: ignore
Turtle().dot(color="blue")  # type: ignore

dot()
dot(10)
dot(size=10)
dot((0, 0, 0))
dot(size=(0, 0, 0))
dot("blue")
dot("")
dot(size="blue")
dot(20, "blue")
dot(20, "blue")
dot(20, (0, 0, 0))
dot(20, 0, 0, 0)

dot(size=10, color="blue")  # type: ignore
dot(10, color="blue")  # type: ignore
dot(color="blue")  # type: ignore



from typing import Any
from typing_extensions import Self

class local:
    __slots__ = (
        "_local__impl",
        "_local_type_set_descriptors",
        "_local_type_get_descriptors",
        "_local_type_vars",
        "_local_type_del_descriptors",
        "_local_type",
        "_local_type_set_or_del_descriptors",
    )
    def __new__(cls, *args: object, **kwargs: object) -> Self: ...
    def __copy__(self) -> Self: ...
    def __getattribute__(self, name: str, /) -> Any: ...
    def __delattr__(self, name: str, /) -> None: ...
    def __setattr__(self, name: str, value: Any, /) -> None: ...

__all__ = ["local"]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.model_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _AutoShardPolicy:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _AutoShardPolicyEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_AutoShardPolicy.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AUTO: _AutoShardPolicy.ValueType  # 0
    """AUTO: Attempts FILE-based sharding, falling back to DATA-based sharding."""
    FILE: _AutoShardPolicy.ValueType  # 1
    """FILE: Shards by input files (i.e. each worker will get a set of files to
    process). When this option is selected, make sure that there is at least as
    many files as workers. If there are fewer input files than workers, a
    runtime error will be raised.
    """
    DATA: _AutoShardPolicy.ValueType  # 2
    """DATA: Shards by elements produced by the dataset. Each worker will process
    the whole dataset and discard the portion that is not for itself. Note that
    for this mode to correctly partitions the dataset elements, the dataset
    needs to produce elements in a deterministic order.
    """
    HINT: _AutoShardPolicy.ValueType  # 3
    """HINT: Looks for the presence of `shard(SHARD_HINT, ...)` which is treated
    as a placeholder to replace with `shard(num_workers, worker_index)`.
    """
    OFF: _AutoShardPolicy.ValueType  # -1
    """OFF: No sharding will be performed."""

class AutoShardPolicy(_AutoShardPolicy, metaclass=_AutoShardPolicyEnumTypeWrapper):
    """Represents the type of auto-sharding we enable."""

AUTO: AutoShardPolicy.ValueType  # 0
"""AUTO: Attempts FILE-based sharding, falling back to DATA-based sharding."""
FILE: AutoShardPolicy.ValueType  # 1
"""FILE: Shards by input files (i.e. each worker will get a set of files to
process). When this option is selected, make sure that there is at least as
many files as workers. If there are fewer input files than workers, a
runtime error will be raised.
"""
DATA: AutoShardPolicy.ValueType  # 2
"""DATA: Shards by elements produced by the dataset. Each worker will process
the whole dataset and discard the portion that is not for itself. Note that
for this mode to correctly partitions the dataset elements, the dataset
needs to produce elements in a deterministic order.
"""
HINT: AutoShardPolicy.ValueType  # 3
"""HINT: Looks for the presence of `shard(SHARD_HINT, ...)` which is treated
as a placeholder to replace with `shard(num_workers, worker_index)`.
"""
OFF: AutoShardPolicy.ValueType  # -1
"""OFF: No sharding will be performed."""
global___AutoShardPolicy = AutoShardPolicy

class _ExternalStatePolicy:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ExternalStatePolicyEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ExternalStatePolicy.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    POLICY_WARN: _ExternalStatePolicy.ValueType  # 0
    POLICY_IGNORE: _ExternalStatePolicy.ValueType  # 1
    POLICY_FAIL: _ExternalStatePolicy.ValueType  # 2

class ExternalStatePolicy(_ExternalStatePolicy, metaclass=_ExternalStatePolicyEnumTypeWrapper):
    """Represents how to handle external state during serialization."""

POLICY_WARN: ExternalStatePolicy.ValueType  # 0
POLICY_IGNORE: ExternalStatePolicy.ValueType  # 1
POLICY_FAIL: ExternalStatePolicy.ValueType  # 2
global___ExternalStatePolicy = ExternalStatePolicy

@typing.final
class AutotuneOptions(google.protobuf.message.Message):
    """next: 6"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENABLED_FIELD_NUMBER: builtins.int
    CPU_BUDGET_FIELD_NUMBER: builtins.int
    RAM_BUDGET_FIELD_NUMBER: builtins.int
    AUTOTUNE_ALGORITHM_FIELD_NUMBER: builtins.int
    INITIAL_PARALLELISM_FIELD_NUMBER: builtins.int
    enabled: builtins.bool
    cpu_budget: builtins.int
    ram_budget: builtins.int
    autotune_algorithm: tensorflow.core.framework.model_pb2.AutotuneAlgorithm.ValueType
    initial_parallelism: builtins.int
    def __init__(
        self,
        *,
        enabled: builtins.bool | None = ...,
        cpu_budget: builtins.int | None = ...,
        ram_budget: builtins.int | None = ...,
        autotune_algorithm: tensorflow.core.framework.model_pb2.AutotuneAlgorithm.ValueType | None = ...,
        initial_parallelism: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "autotune_algorithm",
            b"autotune_algorithm",
            "cpu_budget",
            b"cpu_budget",
            "enabled",
            b"enabled",
            "initial_parallelism",
            b"initial_parallelism",
            "optional_autotune_algorithm",
            b"optional_autotune_algorithm",
            "optional_cpu_budget",
            b"optional_cpu_budget",
            "optional_enabled",
            b"optional_enabled",
            "optional_initial_parallelism",
            b"optional_initial_parallelism",
            "optional_ram_budget",
            b"optional_ram_budget",
            "ram_budget",
            b"ram_budget",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "autotune_algorithm",
            b"autotune_algorithm",
            "cpu_budget",
            b"cpu_budget",
            "enabled",
            b"enabled",
            "initial_parallelism",
            b"initial_parallelism",
            "optional_autotune_algorithm",
            b"optional_autotune_algorithm",
            "optional_cpu_budget",
            b"optional_cpu_budget",
            "optional_enabled",
            b"optional_enabled",
            "optional_initial_parallelism",
            b"optional_initial_parallelism",
            "optional_ram_budget",
            b"optional_ram_budget",
            "ram_budget",
            b"ram_budget",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_autotune_algorithm", b"optional_autotune_algorithm"]
    ) -> typing.Literal["autotune_algorithm"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_cpu_budget", b"optional_cpu_budget"]
    ) -> typing.Literal["cpu_budget"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_enabled", b"optional_enabled"]
    ) -> typing.Literal["enabled"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_initial_parallelism", b"optional_initial_parallelism"]
    ) -> typing.Literal["initial_parallelism"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_ram_budget", b"optional_ram_budget"]
    ) -> typing.Literal["ram_budget"] | None: ...

global___AutotuneOptions = AutotuneOptions

@typing.final
class CardinalityOptions(google.protobuf.message.Message):
    """next: 2"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _ComputeLevel:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _ComputeLevelEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[CardinalityOptions._ComputeLevel.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        CARDINALITY_COMPUTE_UNSPECIFIED: CardinalityOptions._ComputeLevel.ValueType  # 0
        CARDINALITY_COMPUTE_LOW: CardinalityOptions._ComputeLevel.ValueType  # 1
        """Cardinality will only be computed if it can be determined in a cheap
        manner (ie. without reading from file sources). If the cardinality would
        be nontrivial to compute, Cardinality() will return UNKNOWN_CARDINALITY.
        """
        CARDINALITY_COMPUTE_MODERATE: CardinalityOptions._ComputeLevel.ValueType  # 2
        """Moderate effort will be made to determine cardinality, such as reading
        index data from source files. If significant work is needed to compute
        cardinality (e.g. reading entire source file contents or executing user
        defined functions), Cardinality() will return UNKNOWN_CARDINALITY.
        """

    class ComputeLevel(_ComputeLevel, metaclass=_ComputeLevelEnumTypeWrapper): ...
    CARDINALITY_COMPUTE_UNSPECIFIED: CardinalityOptions.ComputeLevel.ValueType  # 0
    CARDINALITY_COMPUTE_LOW: CardinalityOptions.ComputeLevel.ValueType  # 1
    """Cardinality will only be computed if it can be determined in a cheap
    manner (ie. without reading from file sources). If the cardinality would
    be nontrivial to compute, Cardinality() will return UNKNOWN_CARDINALITY.
    """
    CARDINALITY_COMPUTE_MODERATE: CardinalityOptions.ComputeLevel.ValueType  # 2
    """Moderate effort will be made to determine cardinality, such as reading
    index data from source files. If significant work is needed to compute
    cardinality (e.g. reading entire source file contents or executing user
    defined functions), Cardinality() will return UNKNOWN_CARDINALITY.
    """

    COMPUTE_LEVEL_FIELD_NUMBER: builtins.int
    compute_level: global___CardinalityOptions.ComputeLevel.ValueType
    def __init__(self, *, compute_level: global___CardinalityOptions.ComputeLevel.ValueType | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["compute_level", b"compute_level"]) -> None: ...

global___CardinalityOptions = CardinalityOptions

@typing.final
class DistributeOptions(google.protobuf.message.Message):
    """next: 3"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTO_SHARD_POLICY_FIELD_NUMBER: builtins.int
    NUM_DEVICES_FIELD_NUMBER: builtins.int
    auto_shard_policy: global___AutoShardPolicy.ValueType
    num_devices: builtins.int
    def __init__(
        self, *, auto_shard_policy: global___AutoShardPolicy.ValueType | None = ..., num_devices: builtins.int | None = ...
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["num_devices", b"num_devices", "optional_num_devices", b"optional_num_devices"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "auto_shard_policy",
            b"auto_shard_policy",
            "num_devices",
            b"num_devices",
            "optional_num_devices",
            b"optional_num_devices",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_num_devices", b"optional_num_devices"]
    ) -> typing.Literal["num_devices"] | None: ...

global___DistributeOptions = DistributeOptions

@typing.final
class OptimizationOptions(google.protobuf.message.Message):
    """next: 22"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    APPLY_DEFAULT_OPTIMIZATIONS_FIELD_NUMBER: builtins.int
    FILTER_FUSION_FIELD_NUMBER: builtins.int
    MAP_AND_BATCH_FUSION_FIELD_NUMBER: builtins.int
    MAP_AND_FILTER_FUSION_FIELD_NUMBER: builtins.int
    MAP_FUSION_FIELD_NUMBER: builtins.int
    MAP_PARALLELIZATION_FIELD_NUMBER: builtins.int
    NOOP_ELIMINATION_FIELD_NUMBER: builtins.int
    PARALLEL_BATCH_FIELD_NUMBER: builtins.int
    SHUFFLE_AND_REPEAT_FUSION_FIELD_NUMBER: builtins.int
    FILTER_PARALLELIZATION_FIELD_NUMBER: builtins.int
    INJECT_PREFETCH_FIELD_NUMBER: builtins.int
    SEQ_INTERLEAVE_PREFETCH_FIELD_NUMBER: builtins.int
    apply_default_optimizations: builtins.bool
    filter_fusion: builtins.bool
    map_and_batch_fusion: builtins.bool
    map_and_filter_fusion: builtins.bool
    map_fusion: builtins.bool
    map_parallelization: builtins.bool
    noop_elimination: builtins.bool
    parallel_batch: builtins.bool
    shuffle_and_repeat_fusion: builtins.bool
    filter_parallelization: builtins.bool
    inject_prefetch: builtins.bool
    seq_interleave_prefetch: builtins.bool
    def __init__(
        self,
        *,
        apply_default_optimizations: builtins.bool | None = ...,
        filter_fusion: builtins.bool | None = ...,
        map_and_batch_fusion: builtins.bool | None = ...,
        map_and_filter_fusion: builtins.bool | None = ...,
        map_fusion: builtins.bool | None = ...,
        map_parallelization: builtins.bool | None = ...,
        noop_elimination: builtins.bool | None = ...,
        parallel_batch: builtins.bool | None = ...,
        shuffle_and_repeat_fusion: builtins.bool | None = ...,
        filter_parallelization: builtins.bool | None = ...,
        inject_prefetch: builtins.bool | None = ...,
        seq_interleave_prefetch: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "apply_default_optimizations",
            b"apply_default_optimizations",
            "filter_fusion",
            b"filter_fusion",
            "filter_parallelization",
            b"filter_parallelization",
            "inject_prefetch",
            b"inject_prefetch",
            "map_and_batch_fusion",
            b"map_and_batch_fusion",
            "map_and_filter_fusion",
            b"map_and_filter_fusion",
            "map_fusion",
            b"map_fusion",
            "map_parallelization",
            b"map_parallelization",
            "noop_elimination",
            b"noop_elimination",
            "optional_apply_default_optimizations",
            b"optional_apply_default_optimizations",
            "optional_filter_fusion",
            b"optional_filter_fusion",
            "optional_filter_parallelization",
            b"optional_filter_parallelization",
            "optional_inject_prefetch",
            b"optional_inject_prefetch",
            "optional_map_and_batch_fusion",
            b"optional_map_and_batch_fusion",
            "optional_map_and_filter_fusion",
            b"optional_map_and_filter_fusion",
            "optional_map_fusion",
            b"optional_map_fusion",
            "optional_map_parallelization",
            b"optional_map_parallelization",
            "optional_noop_elimination",
            b"optional_noop_elimination",
            "optional_parallel_batch",
            b"optional_parallel_batch",
            "optional_seq_interleave_prefetch",
            b"optional_seq_interleave_prefetch",
            "optional_shuffle_and_repeat_fusion",
            b"optional_shuffle_and_repeat_fusion",
            "parallel_batch",
            b"parallel_batch",
            "seq_interleave_prefetch",
            b"seq_interleave_prefetch",
            "shuffle_and_repeat_fusion",
            b"shuffle_and_repeat_fusion",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "apply_default_optimizations",
            b"apply_default_optimizations",
            "filter_fusion",
            b"filter_fusion",
            "filter_parallelization",
            b"filter_parallelization",
            "inject_prefetch",
            b"inject_prefetch",
            "map_and_batch_fusion",
            b"map_and_batch_fusion",
            "map_and_filter_fusion",
            b"map_and_filter_fusion",
            "map_fusion",
            b"map_fusion",
            "map_parallelization",
            b"map_parallelization",
            "noop_elimination",
            b"noop_elimination",
            "optional_apply_default_optimizations",
            b"optional_apply_default_optimizations",
            "optional_filter_fusion",
            b"optional_filter_fusion",
            "optional_filter_parallelization",
            b"optional_filter_parallelization",
            "optional_inject_prefetch",
            b"optional_inject_prefetch",
            "optional_map_and_batch_fusion",
            b"optional_map_and_batch_fusion",
            "optional_map_and_filter_fusion",
            b"optional_map_and_filter_fusion",
            "optional_map_fusion",
            b"optional_map_fusion",
            "optional_map_parallelization",
            b"optional_map_parallelization",
            "optional_noop_elimination",
            b"optional_noop_elimination",
            "optional_parallel_batch",
            b"optional_parallel_batch",
            "optional_seq_interleave_prefetch",
            b"optional_seq_interleave_prefetch",
            "optional_shuffle_and_repeat_fusion",
            b"optional_shuffle_and_repeat_fusion",
            "parallel_batch",
            b"parallel_batch",
            "seq_interleave_prefetch",
            b"seq_interleave_prefetch",
            "shuffle_and_repeat_fusion",
            b"shuffle_and_repeat_fusion",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_apply_default_optimizations", b"optional_apply_default_optimizations"]
    ) -> typing.Literal["apply_default_optimizations"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_filter_fusion", b"optional_filter_fusion"]
    ) -> typing.Literal["filter_fusion"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_filter_parallelization", b"optional_filter_parallelization"]
    ) -> typing.Literal["filter_parallelization"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_inject_prefetch", b"optional_inject_prefetch"]
    ) -> typing.Literal["inject_prefetch"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_map_and_batch_fusion", b"optional_map_and_batch_fusion"]
    ) -> typing.Literal["map_and_batch_fusion"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_map_and_filter_fusion", b"optional_map_and_filter_fusion"]
    ) -> typing.Literal["map_and_filter_fusion"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_map_fusion", b"optional_map_fusion"]
    ) -> typing.Literal["map_fusion"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_map_parallelization", b"optional_map_parallelization"]
    ) -> typing.Literal["map_parallelization"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_noop_elimination", b"optional_noop_elimination"]
    ) -> typing.Literal["noop_elimination"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_parallel_batch", b"optional_parallel_batch"]
    ) -> typing.Literal["parallel_batch"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_seq_interleave_prefetch", b"optional_seq_interleave_prefetch"]
    ) -> typing.Literal["seq_interleave_prefetch"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_shuffle_and_repeat_fusion", b"optional_shuffle_and_repeat_fusion"]
    ) -> typing.Literal["shuffle_and_repeat_fusion"] | None: ...

global___OptimizationOptions = OptimizationOptions

@typing.final
class ServiceOptions(google.protobuf.message.Message):
    """next: 2"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PINNED_FIELD_NUMBER: builtins.int
    pinned: builtins.bool
    def __init__(self, *, pinned: builtins.bool | None = ...) -> None: ...
    def HasField(
        self, field_name: typing.Literal["optional_pinned", b"optional_pinned", "pinned", b"pinned"]
    ) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["optional_pinned", b"optional_pinned", "pinned", b"pinned"]) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_pinned", b"optional_pinned"]
    ) -> typing.Literal["pinned"] | None: ...

global___ServiceOptions = ServiceOptions

@typing.final
class ThreadingOptions(google.protobuf.message.Message):
    """next: 3"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAX_INTRA_OP_PARALLELISM_FIELD_NUMBER: builtins.int
    PRIVATE_THREADPOOL_SIZE_FIELD_NUMBER: builtins.int
    max_intra_op_parallelism: builtins.int
    private_threadpool_size: builtins.int
    def __init__(
        self, *, max_intra_op_parallelism: builtins.int | None = ..., private_threadpool_size: builtins.int | None = ...
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "max_intra_op_parallelism",
            b"max_intra_op_parallelism",
            "optional_max_intra_op_parallelism",
            b"optional_max_intra_op_parallelism",
            "optional_private_threadpool_size",
            b"optional_private_threadpool_size",
            "private_threadpool_size",
            b"private_threadpool_size",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "max_intra_op_parallelism",
            b"max_intra_op_parallelism",
            "optional_max_intra_op_parallelism",
            b"optional_max_intra_op_parallelism",
            "optional_private_threadpool_size",
            b"optional_private_threadpool_size",
            "private_threadpool_size",
            b"private_threadpool_size",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_max_intra_op_parallelism", b"optional_max_intra_op_parallelism"]
    ) -> typing.Literal["max_intra_op_parallelism"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_private_threadpool_size", b"optional_private_threadpool_size"]
    ) -> typing.Literal["private_threadpool_size"] | None: ...

global___ThreadingOptions = ThreadingOptions

@typing.final
class Options(google.protobuf.message.Message):
    """Message stored with Dataset objects to control how datasets are processed and
    optimized.

    next: 13
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATASET_NAME_FIELD_NUMBER: builtins.int
    FRAMEWORK_TYPE_FIELD_NUMBER: builtins.int
    DETERMINISTIC_FIELD_NUMBER: builtins.int
    AUTOTUNE_OPTIONS_FIELD_NUMBER: builtins.int
    DISTRIBUTE_OPTIONS_FIELD_NUMBER: builtins.int
    OPTIMIZATION_OPTIONS_FIELD_NUMBER: builtins.int
    SERVICE_OPTIONS_FIELD_NUMBER: builtins.int
    SLACK_FIELD_NUMBER: builtins.int
    THREADING_OPTIONS_FIELD_NUMBER: builtins.int
    EXTERNAL_STATE_POLICY_FIELD_NUMBER: builtins.int
    SYMBOLIC_CHECKPOINT_FIELD_NUMBER: builtins.int
    WARM_START_FIELD_NUMBER: builtins.int
    dataset_name: builtins.str
    deterministic: builtins.bool
    slack: builtins.bool
    external_state_policy: global___ExternalStatePolicy.ValueType
    symbolic_checkpoint: builtins.bool
    warm_start: builtins.bool
    @property
    def framework_type(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of frameworks used to generate this dataset."""

    @property
    def autotune_options(self) -> global___AutotuneOptions:
        """The autotune options associated with the dataset."""

    @property
    def distribute_options(self) -> global___DistributeOptions:
        """The distribution strategy options associated with the dataset."""

    @property
    def optimization_options(self) -> global___OptimizationOptions:
        """The optimization options associated with the dataset."""

    @property
    def service_options(self) -> global___ServiceOptions:
        """The tf.data service options associated with the dataset."""

    @property
    def threading_options(self) -> global___ThreadingOptions:
        """The threading options associated with the dataset."""

    def __init__(
        self,
        *,
        dataset_name: builtins.str | None = ...,
        framework_type: collections.abc.Iterable[builtins.str] | None = ...,
        deterministic: builtins.bool | None = ...,
        autotune_options: global___AutotuneOptions | None = ...,
        distribute_options: global___DistributeOptions | None = ...,
        optimization_options: global___OptimizationOptions | None = ...,
        service_options: global___ServiceOptions | None = ...,
        slack: builtins.bool | None = ...,
        threading_options: global___ThreadingOptions | None = ...,
        external_state_policy: global___ExternalStatePolicy.ValueType | None = ...,
        symbolic_checkpoint: builtins.bool | None = ...,
        warm_start: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "autotune_options",
            b"autotune_options",
            "dataset_name",
            b"dataset_name",
            "deterministic",
            b"deterministic",
            "distribute_options",
            b"distribute_options",
            "external_state_policy",
            b"external_state_policy",
            "optimization_options",
            b"optimization_options",
            "optional_dataset_name",
            b"optional_dataset_name",
            "optional_deterministic",
            b"optional_deterministic",
            "optional_external_state_policy",
            b"optional_external_state_policy",
            "optional_slack",
            b"optional_slack",
            "optional_symbolic_checkpoint",
            b"optional_symbolic_checkpoint",
            "optional_warm_start",
            b"optional_warm_start",
            "service_options",
            b"service_options",
            "slack",
            b"slack",
            "symbolic_checkpoint",
            b"symbolic_checkpoint",
            "threading_options",
            b"threading_options",
            "warm_start",
            b"warm_start",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "autotune_options",
            b"autotune_options",
            "dataset_name",
            b"dataset_name",
            "deterministic",
            b"deterministic",
            "distribute_options",
            b"distribute_options",
            "external_state_policy",
            b"external_state_policy",
            "framework_type",
            b"framework_type",
            "optimization_options",
            b"optimization_options",
            "optional_dataset_name",
            b"optional_dataset_name",
            "optional_deterministic",
            b"optional_deterministic",
            "optional_external_state_policy",
            b"optional_external_state_policy",
            "optional_slack",
            b"optional_slack",
            "optional_symbolic_checkpoint",
            b"optional_symbolic_checkpoint",
            "optional_warm_start",
            b"optional_warm_start",
            "service_options",
            b"service_options",
            "slack",
            b"slack",
            "symbolic_checkpoint",
            b"symbolic_checkpoint",
            "threading_options",
            b"threading_options",
            "warm_start",
            b"warm_start",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_dataset_name", b"optional_dataset_name"]
    ) -> typing.Literal["dataset_name"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_deterministic", b"optional_deterministic"]
    ) -> typing.Literal["deterministic"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_external_state_policy", b"optional_external_state_policy"]
    ) -> typing.Literal["external_state_policy"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["optional_slack", b"optional_slack"]) -> typing.Literal["slack"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_symbolic_checkpoint", b"optional_symbolic_checkpoint"]
    ) -> typing.Literal["symbolic_checkpoint"] | None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["optional_warm_start", b"optional_warm_start"]
    ) -> typing.Literal["warm_start"] | None: ...

global___Options = Options



from builtins import bool as _bool
from collections.abc import Iterable
from typing import Literal, overload

import tensorflow as tf
from tensorflow import RaggedTensor, Tensor, norm as norm
from tensorflow._aliases import DTypeLike, IntArray, Integer, ScalarTensorCompatible, TensorCompatible
from tensorflow.math import l2_normalize as l2_normalize

@overload
def matmul(
    a: TensorCompatible,
    b: TensorCompatible,
    transpose_a: _bool = False,
    transpose_b: _bool = False,
    adjoint_a: _bool = False,
    adjoint_b: _bool = False,
    a_is_sparse: _bool = False,
    b_is_sparse: _bool = False,
    output_type: DTypeLike | None = None,
    grad_a: _bool = False,
    grad_b: _bool = False,
    name: str | None = None,
) -> Tensor: ...
@overload
def matmul(
    a: RaggedTensor,
    b: RaggedTensor,
    transpose_a: _bool = False,
    transpose_b: _bool = False,
    adjoint_a: _bool = False,
    adjoint_b: _bool = False,
    a_is_sparse: _bool = False,
    b_is_sparse: _bool = False,
    output_type: DTypeLike | None = None,
    grad_a: _bool = False,
    grad_b: _bool = False,
    name: str | None = None,
) -> RaggedTensor: ...
def set_diag(
    input: TensorCompatible,
    diagonal: TensorCompatible,
    name: str | None = "set_diag",
    k: int = 0,
    align: Literal["RIGHT_LEFT", "RIGHT_RIGHT", "LEFT_LEFT", "LEFT_RIGHT"] = "RIGHT_LEFT",
) -> Tensor: ...
def eye(
    num_rows: ScalarTensorCompatible,
    num_columns: ScalarTensorCompatible | None = None,
    batch_shape: Iterable[int] | IntArray | tf.Tensor | None = None,
    dtype: DTypeLike = ...,
    name: str | None = None,
) -> Tensor: ...
def band_part(input: TensorCompatible, num_lower: Integer, num_upper: Integer, name: str | None = None) -> Tensor: ...
def __getattr__(name: str): ...  # incomplete module



import networkx as nx
from networkx.classes.graph import Graph, _Node

__all__ = ["tree_broadcast_center", "tree_broadcast_time"]

@nx._dispatchable
def tree_broadcast_center(G: Graph[_Node]) -> tuple[int, set[_Node]]: ...
@nx._dispatchable
def tree_broadcast_time(G: Graph[_Node], node: int | None = None) -> int: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.allocation_description_pb2
import tensorflow.core.framework.tensor_description_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class AllocationRecord(google.protobuf.message.Message):
    """An allocation/de-allocation operation performed by the allocator."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALLOC_MICROS_FIELD_NUMBER: builtins.int
    ALLOC_BYTES_FIELD_NUMBER: builtins.int
    alloc_micros: builtins.int
    """The timestamp of the operation."""
    alloc_bytes: builtins.int
    """Number of bytes allocated, or de-allocated if negative."""
    def __init__(self, *, alloc_micros: builtins.int | None = ..., alloc_bytes: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["alloc_bytes", b"alloc_bytes", "alloc_micros", b"alloc_micros"]) -> None: ...

global___AllocationRecord = AllocationRecord

@typing.final
class AllocatorMemoryUsed(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALLOCATOR_NAME_FIELD_NUMBER: builtins.int
    TOTAL_BYTES_FIELD_NUMBER: builtins.int
    PEAK_BYTES_FIELD_NUMBER: builtins.int
    LIVE_BYTES_FIELD_NUMBER: builtins.int
    ALLOCATION_RECORDS_FIELD_NUMBER: builtins.int
    ALLOCATOR_BYTES_IN_USE_FIELD_NUMBER: builtins.int
    allocator_name: builtins.str
    total_bytes: builtins.int
    """These are per-node allocator memory stats."""
    peak_bytes: builtins.int
    live_bytes: builtins.int
    """The bytes that are not deallocated."""
    allocator_bytes_in_use: builtins.int
    """These are snapshots of the overall allocator memory stats.
    The number of live bytes currently allocated by the allocator.
    """
    @property
    def allocation_records(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AllocationRecord]:
        """The allocation and deallocation timeline."""

    def __init__(
        self,
        *,
        allocator_name: builtins.str | None = ...,
        total_bytes: builtins.int | None = ...,
        peak_bytes: builtins.int | None = ...,
        live_bytes: builtins.int | None = ...,
        allocation_records: collections.abc.Iterable[global___AllocationRecord] | None = ...,
        allocator_bytes_in_use: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allocation_records",
            b"allocation_records",
            "allocator_bytes_in_use",
            b"allocator_bytes_in_use",
            "allocator_name",
            b"allocator_name",
            "live_bytes",
            b"live_bytes",
            "peak_bytes",
            b"peak_bytes",
            "total_bytes",
            b"total_bytes",
        ],
    ) -> None: ...

global___AllocatorMemoryUsed = AllocatorMemoryUsed

@typing.final
class NodeOutput(google.protobuf.message.Message):
    """Output sizes recorded for a single execution of a graph node."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SLOT_FIELD_NUMBER: builtins.int
    TENSOR_DESCRIPTION_FIELD_NUMBER: builtins.int
    slot: builtins.int
    @property
    def tensor_description(self) -> tensorflow.core.framework.tensor_description_pb2.TensorDescription: ...
    def __init__(
        self,
        *,
        slot: builtins.int | None = ...,
        tensor_description: tensorflow.core.framework.tensor_description_pb2.TensorDescription | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["tensor_description", b"tensor_description"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["slot", b"slot", "tensor_description", b"tensor_description"]) -> None: ...

global___NodeOutput = NodeOutput

@typing.final
class MemoryStats(google.protobuf.message.Message):
    """For memory tracking."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TEMP_MEMORY_SIZE_FIELD_NUMBER: builtins.int
    PERSISTENT_MEMORY_SIZE_FIELD_NUMBER: builtins.int
    PERSISTENT_TENSOR_ALLOC_IDS_FIELD_NUMBER: builtins.int
    DEVICE_TEMP_MEMORY_SIZE_FIELD_NUMBER: builtins.int
    DEVICE_PERSISTENT_MEMORY_SIZE_FIELD_NUMBER: builtins.int
    DEVICE_PERSISTENT_TENSOR_ALLOC_IDS_FIELD_NUMBER: builtins.int
    temp_memory_size: builtins.int
    persistent_memory_size: builtins.int
    device_temp_memory_size: builtins.int
    device_persistent_memory_size: builtins.int
    @property
    def persistent_tensor_alloc_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def device_persistent_tensor_alloc_ids(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        temp_memory_size: builtins.int | None = ...,
        persistent_memory_size: builtins.int | None = ...,
        persistent_tensor_alloc_ids: collections.abc.Iterable[builtins.int] | None = ...,
        device_temp_memory_size: builtins.int | None = ...,
        device_persistent_memory_size: builtins.int | None = ...,
        device_persistent_tensor_alloc_ids: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "device_persistent_memory_size",
            b"device_persistent_memory_size",
            "device_persistent_tensor_alloc_ids",
            b"device_persistent_tensor_alloc_ids",
            "device_temp_memory_size",
            b"device_temp_memory_size",
            "persistent_memory_size",
            b"persistent_memory_size",
            "persistent_tensor_alloc_ids",
            b"persistent_tensor_alloc_ids",
            "temp_memory_size",
            b"temp_memory_size",
        ],
    ) -> None: ...

global___MemoryStats = MemoryStats

@typing.final
class NodeExecStats(google.protobuf.message.Message):
    """Time/size stats recorded for a single execution of a graph node."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_NAME_FIELD_NUMBER: builtins.int
    ALL_START_MICROS_FIELD_NUMBER: builtins.int
    OP_START_REL_MICROS_FIELD_NUMBER: builtins.int
    OP_END_REL_MICROS_FIELD_NUMBER: builtins.int
    ALL_END_REL_MICROS_FIELD_NUMBER: builtins.int
    MEMORY_FIELD_NUMBER: builtins.int
    OUTPUT_FIELD_NUMBER: builtins.int
    TIMELINE_LABEL_FIELD_NUMBER: builtins.int
    SCHEDULED_MICROS_FIELD_NUMBER: builtins.int
    THREAD_ID_FIELD_NUMBER: builtins.int
    REFERENCED_TENSOR_FIELD_NUMBER: builtins.int
    MEMORY_STATS_FIELD_NUMBER: builtins.int
    ALL_START_NANOS_FIELD_NUMBER: builtins.int
    OP_START_REL_NANOS_FIELD_NUMBER: builtins.int
    OP_END_REL_NANOS_FIELD_NUMBER: builtins.int
    ALL_END_REL_NANOS_FIELD_NUMBER: builtins.int
    SCHEDULED_NANOS_FIELD_NUMBER: builtins.int
    node_name: builtins.str
    """TODO(tucker): Use some more compact form of node identity than
    the full string name.  Either all processes should agree on a
    global id (cost_id?) for each node, or we should use a hash of
    the name.
    """
    all_start_micros: builtins.int
    op_start_rel_micros: builtins.int
    op_end_rel_micros: builtins.int
    all_end_rel_micros: builtins.int
    timeline_label: builtins.str
    scheduled_micros: builtins.int
    thread_id: builtins.int
    all_start_nanos: builtins.int
    op_start_rel_nanos: builtins.int
    op_end_rel_nanos: builtins.int
    all_end_rel_nanos: builtins.int
    scheduled_nanos: builtins.int
    @property
    def memory(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___AllocatorMemoryUsed]: ...
    @property
    def output(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NodeOutput]: ...
    @property
    def referenced_tensor(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.framework.allocation_description_pb2.AllocationDescription
    ]: ...
    @property
    def memory_stats(self) -> global___MemoryStats: ...
    def __init__(
        self,
        *,
        node_name: builtins.str | None = ...,
        all_start_micros: builtins.int | None = ...,
        op_start_rel_micros: builtins.int | None = ...,
        op_end_rel_micros: builtins.int | None = ...,
        all_end_rel_micros: builtins.int | None = ...,
        memory: collections.abc.Iterable[global___AllocatorMemoryUsed] | None = ...,
        output: collections.abc.Iterable[global___NodeOutput] | None = ...,
        timeline_label: builtins.str | None = ...,
        scheduled_micros: builtins.int | None = ...,
        thread_id: builtins.int | None = ...,
        referenced_tensor: (
            collections.abc.Iterable[tensorflow.core.framework.allocation_description_pb2.AllocationDescription] | None
        ) = ...,
        memory_stats: global___MemoryStats | None = ...,
        all_start_nanos: builtins.int | None = ...,
        op_start_rel_nanos: builtins.int | None = ...,
        op_end_rel_nanos: builtins.int | None = ...,
        all_end_rel_nanos: builtins.int | None = ...,
        scheduled_nanos: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["memory_stats", b"memory_stats"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "all_end_rel_micros",
            b"all_end_rel_micros",
            "all_end_rel_nanos",
            b"all_end_rel_nanos",
            "all_start_micros",
            b"all_start_micros",
            "all_start_nanos",
            b"all_start_nanos",
            "memory",
            b"memory",
            "memory_stats",
            b"memory_stats",
            "node_name",
            b"node_name",
            "op_end_rel_micros",
            b"op_end_rel_micros",
            "op_end_rel_nanos",
            b"op_end_rel_nanos",
            "op_start_rel_micros",
            b"op_start_rel_micros",
            "op_start_rel_nanos",
            b"op_start_rel_nanos",
            "output",
            b"output",
            "referenced_tensor",
            b"referenced_tensor",
            "scheduled_micros",
            b"scheduled_micros",
            "scheduled_nanos",
            b"scheduled_nanos",
            "thread_id",
            b"thread_id",
            "timeline_label",
            b"timeline_label",
        ],
    ) -> None: ...

global___NodeExecStats = NodeExecStats

@typing.final
class DeviceStepStats(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ThreadNamesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.str
        def __init__(self, *, key: builtins.int | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    DEVICE_FIELD_NUMBER: builtins.int
    NODE_STATS_FIELD_NUMBER: builtins.int
    THREAD_NAMES_FIELD_NUMBER: builtins.int
    device: builtins.str
    @property
    def node_stats(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NodeExecStats]: ...
    @property
    def thread_names(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.str]:
        """Its key is thread id."""

    def __init__(
        self,
        *,
        device: builtins.str | None = ...,
        node_stats: collections.abc.Iterable[global___NodeExecStats] | None = ...,
        thread_names: collections.abc.Mapping[builtins.int, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["device", b"device", "node_stats", b"node_stats", "thread_names", b"thread_names"]
    ) -> None: ...

global___DeviceStepStats = DeviceStepStats

@typing.final
class StepStats(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEV_STATS_FIELD_NUMBER: builtins.int
    @property
    def dev_stats(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DeviceStepStats]: ...
    def __init__(self, *, dev_stats: collections.abc.Iterable[global___DeviceStepStats] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["dev_stats", b"dev_stats"]) -> None: ...

global___StepStats = StepStats



from _typeshed import Incomplete

__all__ = [
    "no_filter",
    "hide_nodes",
    "hide_edges",
    "hide_multiedges",
    "hide_diedges",
    "hide_multidiedges",
    "show_nodes",
    "show_edges",
    "show_multiedges",
    "show_diedges",
    "show_multidiedges",
]

def no_filter(*items): ...
def hide_nodes(nodes): ...
def hide_diedges(edges): ...
def hide_edges(edges): ...
def hide_multidiedges(edges): ...
def hide_multiedges(edges): ...

class show_nodes:
    nodes: Incomplete
    def __init__(self, nodes) -> None: ...
    def __call__(self, node): ...

def show_diedges(edges): ...
def show_edges(edges): ...
def show_multidiedges(edges): ...
def show_multiedges(edges): ...



from enum import Enum
from typing import Final

class Domain(Enum):
    GOOGLE_INTERNAL = 1
    PUBLIC = 2

OSS_DOMAIN: Final[Domain]
OSS_MAJOR: Final[int]
OSS_MINOR: Final[int]
OSS_PATCH: Final[int]
OSS_SUFFIX: Final[str]
DOMAIN: Final[Domain]
MAJOR: Final[int]
MINOR: Final[int]
PATCH: Final[int]
SUFFIX: Final[str]

class VersionError(Exception): ...

def ValidateProtobufRuntimeVersion(
    gen_domain: Domain, gen_major: int, gen_minor: int, gen_patch: int, gen_suffix: str, location: str
) -> None: ...



from typing import Final

__version__: Final[str]
A0: Final[tuple[float, float]]
A1: Final[tuple[float, float]]
A2: Final[tuple[float, float]]
A3: Final[tuple[float, float]]
A4: Final[tuple[float, float]]
A5: Final[tuple[float, float]]
A6: Final[tuple[float, float]]
A7: Final[tuple[float, float]]
A8: Final[tuple[float, float]]
A9: Final[tuple[float, float]]
A10: Final[tuple[float, float]]
B0: Final[tuple[float, float]]
B1: Final[tuple[float, float]]
B2: Final[tuple[float, float]]
B3: Final[tuple[float, float]]
B4: Final[tuple[float, float]]
B5: Final[tuple[float, float]]
B6: Final[tuple[float, float]]
B7: Final[tuple[float, float]]
B8: Final[tuple[float, float]]
B9: Final[tuple[float, float]]
B10: Final[tuple[float, float]]
C0: Final[tuple[float, float]]
C1: Final[tuple[float, float]]
C2: Final[tuple[float, float]]
C3: Final[tuple[float, float]]
C4: Final[tuple[float, float]]
C5: Final[tuple[float, float]]
C6: Final[tuple[float, float]]
C7: Final[tuple[float, float]]
C8: Final[tuple[float, float]]
C9: Final[tuple[float, float]]
C10: Final[tuple[float, float]]
LETTER: Final[tuple[float, float]]
LEGAL: Final[tuple[float, float]]
ELEVENSEVENTEEN: Final[tuple[float, float]]
JUNIOR_LEGAL: Final[tuple[float, float]]
HALF_LETTER: Final[tuple[float, float]]
GOV_LETTER: Final[tuple[float, float]]
GOV_LEGAL: Final[tuple[float, float]]
TABLOID: Final[tuple[float, float]]
LEDGER: Final[tuple[float, float]]
letter: Final[tuple[float, float]]
legal: Final[tuple[float, float]]
elevenSeventeen: Final[tuple[float, float]]

def landscape(pagesize: tuple[float, float]) -> tuple[float, float]: ...
def portrait(pagesize: tuple[float, float]) -> tuple[float, float]: ...



from collections.abc import Collection, Sequence
from pathlib import Path
from typing import Generic, Literal, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias

import numpy as np
import numpy.typing as npt

from . import _EncodedRLE

PYTHON_VERSION: int

@type_check_only
class _Image(TypedDict):
    id: int
    width: int
    height: int
    file_name: str

_TPolygonSegmentation: TypeAlias = list[list[float]]

@type_check_only
class _RLE(TypedDict):
    size: list[int]
    counts: list[int]

@type_check_only
class _Annotation(TypedDict):
    id: int
    image_id: int
    category_id: int
    segmentation: _TPolygonSegmentation | _RLE | _EncodedRLE
    area: float
    bbox: list[float]
    iscrowd: int

_TSeg = TypeVar("_TSeg", _TPolygonSegmentation, _RLE, _EncodedRLE)

@type_check_only
class _AnnotationG(TypedDict, Generic[_TSeg]):
    id: int
    image_id: int
    category_id: int
    segmentation: _TSeg
    area: float
    bbox: list[float]
    iscrowd: int

@type_check_only
class _Category(TypedDict):
    id: int
    name: str
    supercategory: str

@type_check_only
class _Dataset(TypedDict):
    images: list[_Image]
    annotations: list[_Annotation]
    categories: list[_Category]

class COCO:
    anns: dict[int, _Annotation]
    dataset: _Dataset
    cats: dict[int, _Category]
    imgs: dict[int, _Image]
    imgToAnns: dict[int, list[_Annotation]]
    catToImgs: dict[int, list[int]]
    def __init__(self, annotation_file: str | Path | None = None) -> None: ...
    def createIndex(self) -> None: ...
    def info(self) -> None: ...
    def getAnnIds(
        self,
        imgIds: Collection[int] | int = [],
        catIds: Collection[int] | int = [],
        areaRng: Sequence[float] = [],
        iscrowd: bool | None = None,
    ) -> list[int]: ...
    def getCatIds(
        self, catNms: Collection[str] | str = [], supNms: Collection[str] | str = [], catIds: Collection[int] | int = []
    ) -> list[int]: ...
    def getImgIds(self, imgIds: Collection[int] | int = [], catIds: list[int] | int = []) -> list[int]: ...
    def loadAnns(self, ids: Collection[int] | int = []) -> list[_Annotation]: ...
    def loadCats(self, ids: Collection[int] | int = []) -> list[_Category]: ...
    def loadImgs(self, ids: Collection[int] | int = []) -> list[_Image]: ...
    def showAnns(self, anns: Sequence[_Annotation], draw_bbox: bool = False) -> None: ...
    def loadRes(self, resFile: str) -> COCO: ...
    def download(self, tarDir: str | None = None, imgIds: Collection[int] = []) -> Literal[-1] | None: ...
    def loadNumpyAnnotations(self, data: npt.NDArray[np.float64]) -> list[_Annotation]: ...
    @overload
    def annToRLE(self, ann: _AnnotationG[_RLE]) -> _RLE: ...
    @overload
    def annToRLE(self, ann: _AnnotationG[_EncodedRLE]) -> _EncodedRLE: ...
    @overload
    def annToRLE(self, ann: _AnnotationG[_TPolygonSegmentation]) -> _EncodedRLE: ...
    def annToMask(self, ann: _Annotation) -> npt.NDArray[np.uint8]: ...
